/***************************************************************************
 *   Copyright (C) 2004 by Sayantam Dey                                    *
 *   sayantam@gmail.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <kglobalsettings.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <kstandarddirs.h>
#include <kdebug.h>
#include <kwin.h>
#include <kurldrag.h>
#include <kmessagebox.h>
#include <klocale.h>

#include <qpixmap.h>
#include <qimage.h>
#include <qnamespace.h>

#include "krawlconfig.h"
#include "dropwindow.h"

DropWindow::DropWindow(WId _container_id, int x, int y, QWidget *parent, const char *name)
 : QWidget(parent, name)
{
	if (!name)
		this->setName("drop window");
	m_container_id = _container_id;
	setFocusPolicy(NoFocus);
	QPixmap pix = QPixmap(locate("data", "krawlsitepart/pics/drop_win.png"));
	mask = *pix.mask();
	this->setBackgroundPixmap(pix);
	this->setMask(mask);
	KWin::setState(winId(), NET::StaysOnTop | NET::SkipTaskbar | NET::SkipPager);	
	this->setAcceptDrops(true);
	createPopupMenu();
	oldX = x;
	oldY = y;
	if (x != -1 && y != -1)
		this->move(x,y);
	this->resize(128,128);
}

DropWindow::~DropWindow()
{
	debugMsg("DropWindow destructor called");
}

void DropWindow::mousePressEvent(QMouseEvent* e)
{
	if (e->button() == LeftButton)
	{
		oldX = oldY = 0;
	}
	else if (e->button() == RightButton)
	{
		int id = m_popup_menu->exec(e->globalPos());
		popupMenuHandler(id);
	}
}

void DropWindow::mouseMoveEvent(QMouseEvent* e)
{
	if (oldX == 0)
	{
		oldX = e->x();
		oldY = e->y();
		return;
	}
	move(x() + (e->x() - oldX), 	y() + (e->y() - oldY));
}

void DropWindow::dragEnterEvent(QDragEnterEvent* _e)
{
	bool a = KURLDrag::canDecode(_e);
// 	debugMsg(QString("dragEnterEvent... kurl::canDecode=%1").arg(a));
	bool b = QTextDrag::canDecode(_e);	
// 	debugMsg(QString("dragEnterEvent... textdrag::canDecode=%1").arg(b));
	_e->accept(a || b);
}

void DropWindow::dropEvent(QDropEvent* _e)
{
	KURL::List urlList;
	QString strUrl;
	if (KURLDrag::decode(_e, urlList))
	{
		QValueListIterator<KURL> it;
		for (it = urlList.begin(); it != urlList.end(); it++)
		{
			KURL d_url = (*it);
			m_urlStrList.append(d_url.url());
		}
		emit droppedItem(urlList);
	}
	else if (QTextDrag::decode(_e, strUrl))
	{
		m_urlStrList.append(strUrl);
		emit droppedItem(strUrl);
	}
}

void DropWindow::createPopupMenu()
{
	m_popup_menu = new KPopupMenu(this, "popup_menu");
	int index = 0;
	m_popup_menu->insertTitle(i18n("Drop Target"), index++);
	m_popup_menu->insertItem(i18n("Show &URLs"), ID_SHOW_URL, index++);
	m_popup_menu->insertItem(i18n("Show &Parent"), ID_PARENT_SHOW, index++);
	m_popup_menu->insertSeparator(index++);
	m_popup_menu->insertItem(i18n("Show on &All Desktops"), ID_ALL_DESKTOPS, index++);
	if (KrawlConfigSkel::chkDTAllDesks())
	{
		KWin::setOnAllDesktops(winId(), true);
		m_popup_menu->setItemChecked(ID_ALL_DESKTOPS, true);
	}
	m_popup_menu->insertSeparator(index++);
	m_popup_menu->insertItem(i18n("&Close"), ID_CLOSE, index++);
}

void DropWindow::popupMenuHandler(int _id)
{
	switch(_id)
	{
		case ID_SHOW_URL:
		{
			KMessageBox::informationList(this, i18n("This is the list of dropped URLs"), m_urlStrList, i18n("Dropped URLs"));
			break;
		}
		case ID_PARENT_SHOW:
		{
			KWin::WindowInfo win_info = KWin::windowInfo(m_container_id);
			if ( ! win_info.isOnCurrentDesktop() )
				KWin::setCurrentDesktop( win_info.desktop() );
			KWin::raiseWindow(m_container_id);
			break;
		}
		case ID_CLOSE:
		{
			emit windowClosed(x(), y());
			close(true);
			break;
		}
		case ID_ALL_DESKTOPS:
		{
			if (! m_popup_menu->isItemChecked(ID_ALL_DESKTOPS))
			{
				KWin::setOnAllDesktops(winId(), true);
				m_popup_menu->setItemChecked(ID_ALL_DESKTOPS, true);
			}
			else
			{
				KWin::setOnAllDesktops(winId(), false);
				m_popup_menu->setItemChecked(ID_ALL_DESKTOPS, false);
			}
			break;
		}
		default:
			break;
	}
}

void DropWindow::debugMsg(QString _msg)
{
	kdDebug(9815578)<<"DropWindow.... "<<_msg<<endl;
}
		
#include "dropwindow.moc"
