/***************************************************************************
 *   Copyright (C) 2004 by Sayantam Dey                                    *
 *   sayantam@gmail.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <kmessagebox.h>
#include <kfiledialog.h>
#include <klocale.h>
#include <kdebug.h>
#include <kiconloader.h>
#include <kfile.h>
#include <krun.h>

#include <qlayout.h>

#include "filemanager.h"
#include "krawlconfig.h"

FileManager::FileManager(QWidget *parent, const char *name)
 : QWidget(parent, name), first_time(true), fm_show_hidden(false)
{
	QVBoxLayout* v_box_layout = new QVBoxLayout(this, 1, 1, "v_box_layout");
	
	// url requester
	m_KUrlReq = new KURLRequester(this, "m_KUrlReq");
	v_box_layout->addWidget(m_KUrlReq);
	m_KUrlReq->setMode(KFile::Directory | KFile::ExistingOnly | KFile::LocalOnly);
	m_KUrlReq->setSizePolicy(QSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed, false));
	connect(m_KUrlReq, SIGNAL(urlSelected (const QString &)), this, SLOT(changeFileView(const QString& )));
	connect(m_KUrlReq, SIGNAL(returnPressed (const QString &)), this, SLOT(changeFileView(const QString& )));
	
	//toolbar
	m_ToolBar = new KToolBar(this, "tool_bar", true, true);
	m_ToolBar->setIconSize(16);
	v_box_layout->addWidget(m_ToolBar);
	
	// file view
	m_fileView = new KDirOperator(m_KUrlReq->url(), this, "file-view");
	m_fileView->setViewConfig(KrawlConfigSkel::self()->config(), "Filebrowser");
	m_fileView->setEnableDirHighlighting(true);
	m_fileView->setMode(KFile::Mode((int)KFile::Files | (int)KFile::Directory));
	m_fileView->setupMenu(KDirOperator::AllActions);
// 	m_fileView->readConfig(KrawlConfigSkel::self()->config(), "Filebrowser");
	m_fileView->setView(KFile::Simple);
	v_box_layout->addWidget(m_fileView);
	
	connect(m_fileView, SIGNAL(fileSelected(const KFileItem* )), this, SLOT(itemExecuted(const KFileItem* ))); 
	connect(m_fileView, SIGNAL(finishedLoading()), this, SLOT(baseDirChanged()));		
	
	createToolBar();
	loadSavedDefaults();
	setSizePolicy( QSizePolicy( QSizePolicy::Minimum, QSizePolicy::Preferred, 0, 0, sizePolicy().hasHeightForWidth() ) );
}


FileManager::~FileManager()
{
}

QString FileManager::baseDlDirectory()
{
	KURL url(m_KUrlReq->url());
	if (url.isEmpty())
	{
		if (KMessageBox::questionYesNo(this, "Base directory not specified. All files will be downloaded to \n" + 
										 KGlobalSettings::documentPath() + "/krawlsite. If this \n" + 
										 "directory doesnt exists, it will be created. Continue?")  == KMessageBox::No)
		{
			return 0;
		}
		else m_KUrlReq->setURL(KGlobalSettings::documentPath() + "/krawlsite");
	}
	if (url.isValid() == false)
	{
		KMessageBox::error(this, "Malformed directory path. Please reset and try again");
		return 0;
	}
	
	return m_KUrlReq->url();
}

KURL FileManager::branchUrl()
{
	return m_fileView->url();
}

void FileManager::itemExecuted(const KFileItem* _file_Item)
{
	if (_file_Item != 0 && _file_Item->isFile())
	{
		KRun::runURL(_file_Item->url(), _file_Item->mimetype(), false, true);
	}
}

void FileManager::loadSavedDefaults()
{
	rememberLastBaseDir = KrawlConfigSkel::chkLastBaseDirUrl();
	if (rememberLastBaseDir)
	{
		m_KUrlReq->setURL(KrawlConfigSkel::lastLocalDirectory());
	}
	else if (first_time) 
	{
		m_KUrlReq->setURL(KrawlConfigSkel::defaultBaseDir());
	}
	m_KUrlReq->fileDialog()->setURL(m_KUrlReq->url());
	
	m_fileView->setShowHiddenFiles( KrawlConfigSkel::showHiddenFiles() );
}

void FileManager::saveSettings()
{
	if (rememberLastBaseDir)
	{
		KrawlConfigSkel::setLastLocalDirectory(m_KUrlReq->url());
	}
// 	m_fileView->writeConfig(KrawlConfigSkel::self()->config(), "File Manager Settings");
}

void FileManager::changeFileView(const QString& _url_str)
{
	KURL k_url(_url_str);
	if (!k_url.isEmpty() && k_url.isValid() && k_url.isLocalFile())
	{
		resetFileView(k_url);
	}
}

void FileManager::createToolBar()
{
	m_fileView->actionCollection()->action("up")->plug(m_ToolBar);
	m_fileView->actionCollection()->action("back")->plug(m_ToolBar);
	m_fileView->actionCollection()->action("forward")->plug(m_ToolBar);
	m_fileView->actionCollection()->action("home")->plug(m_ToolBar);
	m_fileView->actionCollection()->action("reload")->plug(m_ToolBar);
	m_fileView->actionCollection()->action("mkdir")->plug(m_ToolBar);
	m_fileView->actionCollection()->action("separator")->plug(m_ToolBar);
	m_fileView->actionCollection()->action("short view")->plug(m_ToolBar);
	m_fileView->actionCollection()->action("detailed view")->plug(m_ToolBar);
}

void FileManager::debugMsg(QString _str)
{
	kdDebug(9815578)<<"FileManager.... "<<_str<<endl;
}

void FileManager::resetFileView(KURL _base_url)
{
	m_fileView->setURL(_base_url, true);
}

void FileManager::initFileView()
{
	resetFileView(m_KUrlReq->url());
}

void FileManager::baseDirChanged()
{
	m_KUrlReq->setURL(m_fileView->url().url());
}

#include "filemanager.moc"
