/***************************************************************************
 *   Copyright (C) 2004 by Sayantam Dey                                    *
 *   sayantam@gmail.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <kglobalsettings.h>
#include <qlayout.h>

#include "historybrowser.h"

/**
 * 
 * @param parent 
 * @param name 
 * @param _list 
 * @return 
 */
HistoryBrowser::HistoryBrowser(QWidget *parent, const char *name, const QStringList& _list)
 : QWidget(parent, name)
{
	m_history_list = new KListBox(this, "history_urls");
	if (_list.size() != 0)
		m_history_list->insertStringList(_list,0);
	m_history_list->setSelectionMode(KListBox::Multi);
	connect(m_history_list, SIGNAL(executed(QListBoxItem* )), this, SLOT(slotItemExecuted(QListBoxItem* )));
	
	QGridLayout* grid_layout = new QGridLayout(this, 1, 1, 11, 6, "layout1");
	grid_layout->addWidget(m_history_list, 0, 0);
	
	QFont fm_fnt = KGlobalSettings::generalFont();
	QFontMetrics fm( fm_fnt );
	
	int width = fm.width('X') * averageStringLen(_list);
	int height1 = 3*fm.height() * _list.count()/5;
	int height2 = 4*fm.height();
	int height = height1 > height2 ? height1 : height2;
	m_history_list->setFixedSize(width, height);
	
	setMinimumSize(minimumSizeHint());
// 	setMinimumSize( QSize( width, height ) );
}

HistoryBrowser::~HistoryBrowser()
{
}

void HistoryBrowser::slotItemExecuted(QListBoxItem* _item)
{
	emit addUrlString(_item->text());
}

QStringList HistoryBrowser::getSelectedUrls()
{
	QStringList selected_urls;
	for ( uint i = 0; i < m_history_list->count(); i++)
		if ( m_history_list->isSelected(i) )
			selected_urls.append( m_history_list->item(i)->text() );
	
	return selected_urls;
}

uint HistoryBrowser::averageStringLen(const QStringList& _list)
{
	uint avg_len = 0;
	
	QStringList::ConstIterator it;
	for ( it = _list.begin(); it != _list.end(); it++)
	{
		QString url_str = (*it);
		avg_len += url_str.length();
	}
	avg_len = (avg_len == 0)?80:(avg_len/_list.count());
	return avg_len;
}
		
#include "historybrowser.moc"
