/***************************************************************************
*   Copyright (C) 2004 by Sayantam Dey                                    *
*   sayantam@gmail.com                                                    *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include <kprogress.h>
#include <klocale.h>
#include <ksqueezedtextlabel.h>
#include <qprogressbar.h>

#include <qlabel.h>

#include "infopanel.h"
#include "infopanel.moc"

InfoPanel::InfoPanel( QWidget* parent, const char* name, WFlags fl )
		: UICInfoPanel( parent, name, fl )
{
	m_prgsLinks->setFormat("%v/%m");
	m_prgsImages->setFormat("%v/%m");
	m_prgsLinks->setCenterIndicator(false);
	m_prgsImages->setCenterIndicator(false);
	m_lblTotalDlSize->setText("[0]");
	m_lblCurrentSpeed->setText("[0]");
}

InfoPanel::~InfoPanel()
{}

void InfoPanel::init_widget()
{
	m_prgsBar->setProgress(0);
	m_prgsLinks->setCenterIndicator(true);
	m_prgsImages->setCenterIndicator(true);
}

void InfoPanel::setTotalDlSize(unsigned long value)
{
	QString reported_size;
	// value is in bytes.. first see if its less than 1024, report is as bytes
	if (value < 1024)
	{
		reported_size = QString("[%1 B]").arg(value);
	}
	// if its greater than 1024 but less than 1024*1024, report as KB
	else if (value > 1024 && value < 1024*1024)
	{
		reported_size = QString("[%1 KB]" ).arg(value/1024.0, 0, 'f', 2);
	}
	else // just report it as MB
		reported_size = QString("[%1 MB]").arg(value/(1024.0*1024), 0, 'f', 2);
			
	//qApp->lock();
	m_lblTotalDlSize->setText(reported_size);
	//qApp->unlock();
}

void InfoPanel::setDlSpeed(unsigned long value)
{
	QString speed;
	// m_i_dl_total_size is in bytes.. first see if its less than 1024, report is as bytes
	if (value < 1024)
	{
		speed = QString("[%1 B/s]").arg(value);
	}
	// if its greater than 1024 but less than 1024*1024, report as KB
	else if (value > 1024 && value < 1024*1024)
	{
		speed = QString("[%1 KB/s]").arg(value/1024.0, 0, 'f', 2);
	}
	else // just report it as MB
		speed = QString("[%1 MB/s]").arg(value/(1024.0*1024), 0, 'f', 2);
	
	//qApp->lock();
	m_lblCurrentSpeed->setText(speed);
	//qApp->unlock();
}

void InfoPanel::setStatusMsg(QString qstr_status_msg)
{
	//qApp->lock();
	m_lblStatusMsg->setText(qstr_status_msg);
	//qApp->unlock();
}

void InfoPanel::setLinkStats(int index, int _total_steps)
{
	//qApp->lock();
	m_prgsLinks->QProgressBar::setProgress(index, _total_steps);
	//qApp->unlock();
}

void InfoPanel::setImgStats(int index, int _total_steps)
{
	//qApp->lock();
	m_prgsImages->QProgressBar::setProgress(index, _total_steps);
	//qApp->unlock();
}

void InfoPanel::setCurrentDlUrl(QString _qstr_url)
{
	//qApp->lock();
	m_lblCurrentDlUrl->setText(i18n("Downloading " + _qstr_url + "..."));
	//qApp->unlock();
}

void InfoPanel::clearDlLabel()
{	
	//qApp->lock();
	m_lblCurrentDlUrl->clear();
	//qApp->unlock();
}


void InfoPanel::progressUpdate(int percent)
{
	//qApp->lock();
	m_prgsBar->setProgress(percent);
	//qApp->unlock();
}


void InfoPanel::changeKrawlDisp(const QString& _mode_str)
{
	//qApp->lock();	
	m_lblCrawlMode->setText("[" + _mode_str + "]");
	//qApp->unlock();
}

void InfoPanel::setLinkPrgsTotal(int _size)
{
	//qApp->lock();
	m_prgsLinks->QProgressBar::setProgress(0, _size);
	//qApp->unlock();
}

void InfoPanel::setImagePrgsTotal(int _size)
{
	//qApp->lock();
	m_prgsImages->QProgressBar::setProgress(0, _size);
	//qApp->unlock();
}

/*$SPECIALIZATION$*/



