/***************************************************************************
*   Copyright (C) 2004 by Sayantam Dey                                    *
*   sayantam@gmail.com                                                    *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include <kfiledialog.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <kdebug.h>
#include <ktar.h>
#include <kmimetype.h>
#include <kinputdialog.h>

#include <qfile.h>
#include <qfileinfo.h>
#include <qstring.h>
#include <qcursor.h>
#include <qvariant.h>
#include <qtextedit.h>
#include <qlayout.h>

#include "krawlarchiver.h"
#include "krawlconfig.h"

KrawlArchiver::KrawlArchiver(QWidget* parent, KURL _base_url) : QObject()
{
	m_base_dl_dir = _base_url;
	parentWidget = parent;
}


KrawlArchiver::~KrawlArchiver()
{
	debugMsg("KrawlArchiver destructor called");
}

void KrawlArchiver::initialize()
{
		m_file_list.clear();
}

void KrawlArchiver::createArchive()
{
	emit infoMessage(i18n("Creating Archive..."));
	initialize();
	
	if (m_base_dl_dir == (const KURL&)0 || m_base_dl_dir.isEmpty())
	{
		KMessageBox::sorry(parentWidget, i18n("Meekly refusing to archive an empty directory."), "Krawlsite");
		emit infoMessage(i18n("Process Aborted."));
		return;
	}
	searchIndexFile();
}

void KrawlArchiver::continueArchival()
{
	debugMsg("continueArchival...");
	QString index_file_path;
	if (m_index_file_name == "")
	{
		KURL _chosen_index_file = KFileDialog::getSaveFileName(m_base_dl_dir.path(), i18n("*.htm *.html *.shtml|Html Files\n*.*|All Files"), parentWidget, i18n("No index.html found. Please choose an html file as starting point of archive"));
		
		if (_chosen_index_file.isEmpty())
		{
			debugMsg("_chosen_index_file operation cancelled");
			emit infoMessage(i18n("Process Aborted."));
			return;
		}
		
		if (_chosen_index_file.isValid() == false)
		{
			KMessageBox::error(parentWidget, i18n("This does not appear to be a valid file. Can't continue."), i18n("Error"));
			emit infoMessage(i18n("Process Aborted."));
			return;
		}
		
		index_file_path = _chosen_index_file.path();
	}
	else {
		index_file_path = m_index_file_name;
		debugMsg("index_file_path :");
	}
	
	KURL _index_file_url(index_file_path); 
	
	// should check if the chosen file lies inside the branch path
// 	bool isParentOf;
	m_index_file_rel_path = KURL::relativePath(m_base_dl_dir.path(), _index_file_url.path()/*, &isParentOf*/);
	
// 	if (isParentOf == false)
// 	{
// 		KMessageBox::error(parentWidget, "The chosen index file is not in the downloaded directory hierarchy. Cant continue.", "Error");
// 		return;
// 	}

	m_index_file_rel_path = m_index_file_rel_path.remove(0, 1);
	debugMsg("relative path of index.html:");
	
	// formulate the archive name
	QDir _dir(m_base_dl_dir.path());
	QString archiveName = m_base_dl_dir.upURL().path() + _dir.dirName() + ".war";
	debugMsg("base dir cdUp:");
	m_mod_str_index = m_base_dl_dir.upURL().path().length();
	QString archive_filename = _dir.dirName() + ".war";
	m_archive_mod_str = archive_filename + "/" ;
	m_index_file_rel_path = _dir.dirName() + m_index_file_rel_path;
	m_base_dir = _dir;
	
	KURL __save_url = KFileDialog::getSaveFileName(archiveName, i18n("*.war *.tgz|Web Archives"), parentWidget, i18n("Save Archive"));
	
	if (__save_url.isEmpty())
	{
		debugMsg("operation cancelled");
		emit infoMessage(i18n("Process Aborted."));
		return;
	}
	
	if (__save_url.isValid() == false)
	{
		KMessageBox::error(parentWidget, i18n("This does not appear to be a valid file. Can't continue."), i18n("Error"));
		emit infoMessage(i18n("Process Aborted."));
		return;
	}
	
	QFile _file(__save_url.path());
	if (_file.exists() && 
		KMessageBox::warningYesNo(parentWidget, i18n("File exists. Are you sure you want to continue?"), i18n("Warning-File exists")) == KMessageBox::No)
	{
		emit infoMessage(i18n("Process Aborted."));
		return;
	}	
	m_save_url = __save_url;
	
	// move branch directory to some temp location.
	m_tmp_dir = new KTempDir();
	QDir* _tmp_qdir = m_tmp_dir->qDir();
	KURL _tmp_kurl(_tmp_qdir->path());
	m_tmp_url = _tmp_kurl;
	
	emit infoMessage(i18n("Copying files to temporary location..."));
	// copy the contents of the base directory to this temporary location
	copy_job_ptr = KIO::copy(m_base_dl_dir, _tmp_kurl, true);
	connect(copy_job_ptr, SIGNAL(result(KIO::Job* )),
				this, SLOT(slotCopyJob(KIO::Job* )));

}

void KrawlArchiver::slotCopyJob(KIO::Job* _job_ptr)
{
	disconnect(_job_ptr, SIGNAL(result(KIO::Job *)), this, SLOT(slotCopyJob(KIO::Job* )));
	QString temp_url_path = m_tmp_url.path();
	if (temp_url_path.endsWith("/") == false)
		temp_url_path += "/";
	
	temp_url_path = temp_url_path + m_base_dir.dirName();
	m_tmp_url = KURL(temp_url_path);
	getFileList(m_tmp_url);
}

void KrawlArchiver::getFileList(KURL _url)
{
	debugMsg("getFileList called.");
	// recursive directory listing
	m_b_dir_list = true;
	m_b_index_file_search = false;
	m_file_list.clear();
	list_job_ptr = KIO::listRecursive(_url, false, true);
	connect(list_job_ptr, SIGNAL(result(KIO::Job*)), this, SLOT(processLinks(KIO::Job* )));
	connect(list_job_ptr, SIGNAL(entries(KIO::Job*, const KIO::UDSEntryList& )), this, SLOT(slotDirListing(KIO::Job*, const KIO::UDSEntryList& )));
}

void KrawlArchiver::searchIndexFile()
{
	debugMsg("searchIndexFile...");
	emit infoMessage(i18n("Searching for index.html..."));
	m_b_index_file_search = true;
	m_b_done_once = false;
	m_file_list.clear();
	list_job_ptr = KIO::listRecursive(m_base_dl_dir, false, true);
	connect(list_job_ptr, SIGNAL(result(KIO::Job*)), this, SLOT(slotContinueArchival(KIO::Job*)));
	connect(list_job_ptr, SIGNAL(entries(KIO::Job*, const KIO::UDSEntryList& )), this, SLOT(slotDirListing(KIO::Job*, const KIO::UDSEntryList& )));
}

void 	KrawlArchiver::slotDirListing(KIO::Job* _job_ptr, const KIO::UDSEntryList& uds_entry_list)
{
// 	debugMsg("slotDirListing...");
	if (list_job_ptr != _job_ptr)
	{
		debugMsg("kio jobs mangled!");
		return;
	}
	
	KIO::UDSEntryListConstIterator it;
	
	for ( it = uds_entry_list.begin(); it != uds_entry_list.end(); ++it )
	{
		KIO::UDSEntry uds_entry = KIO::UDSEntry(*it);
		KFileItem* file_item = new KFileItem(uds_entry, m_tmp_url, false, true);
		if (m_b_index_file_search)
		{
			if (file_item->name().contains("index") > 0)
			{
				m_file_list.append(file_item);
			}
			else {
				delete file_item;
				file_item = 0;
			}
		}
		else {
			if (file_item->mimetype().compare("text/html") == 0)
			{
				m_file_list.append(file_item);
			}
		}
	}
}

void KrawlArchiver::slotContinueArchival(KIO::Job *_job_ptr)
{
	if (list_job_ptr != _job_ptr)
	{
		debugMsg("kio jobs mangled!");
		return;
	}
	if (m_b_index_file_search && m_b_done_once == false)
	{
		m_b_done_once = true;
		disconnect(_job_ptr, SIGNAL(result(KIO::Job*)), this, SLOT(slotContinueArchival(KIO::Job* )));
		disconnect(_job_ptr, SIGNAL(entries(KIO::Job*, const KIO::UDSEntryList& )), this, SLOT(slotDirListing(KIO::Job*, const KIO::UDSEntryList& )));
		
		m_index_file_name = "";
		if (m_file_list.count() > 1)
		{
			KFileItem* ptr_item = 0;
			QStringList file_qstr_path_list;
			for (ptr_item = m_file_list.first(); ptr_item; ptr_item = m_file_list.next())
			{
				QString 	index_file_path = m_base_dl_dir.path() + ptr_item->name();
				file_qstr_path_list.append(index_file_path);
			}
			bool okClicked = false;
			m_index_file_name = KInputDialog::getItem(i18n("Multiple potential starting points:"), i18n("The starting point of a web archive is an index.html file. \nMultiple index.html were found. Please select one. \nIf the file you wish to select is not present, press cancel to select from the file dialog."), file_qstr_path_list, 0, false, &okClicked, parentWidget, "file_chooser");
			if (okClicked == false)
			{
				 m_index_file_name = "";
			 }
		 }
		 else if (m_file_list.count() == 1)
		 {
			 m_index_file_name = m_base_dl_dir.path() + m_file_list.at(0)->name();
			 if (KrawlConfigSkel::chkAlwaysAskTarIndex())
			 {
				 bool okClicked = false;
				 QStringList str_list_file(m_index_file_name);
				 m_index_file_name = KInputDialog::getItem(i18n("Potential starting point"), i18n("This is the index.html found. If you do not wish this to be the starting\npoint of your archive, press cancel to select from file dialog"), str_list_file, 0, false, &okClicked, parentWidget, "file_chooser");
				 if (okClicked == false)
				 {
					 m_index_file_name = "";
				 }
			 }
		 }
		continueArchival();
	}
}

void KrawlArchiver::processLinks(KIO::Job* _job_ptr)
{
	disconnect(_job_ptr, SIGNAL(result(KIO::Job*)), this, SLOT(processLinks(KIO::Job* )));
	disconnect(_job_ptr, SIGNAL(entries(KIO::Job*, const KIO::UDSEntryList& )), this, SLOT(slotDirListing(KIO::Job*, const KIO::UDSEntryList& )));
	emit infoMessage(i18n("Processing html links..."));
	if (m_b_dir_list)
	{
		debugMsg("process links...");
		m_b_dir_list = false;
		//////////////////////////////////////////////////////////////////////////////////////////
		Krawler _krawl_inst;
		KFileItem* _ptr_item;
// 		m_archive_mod_str = "/" + m_archive_mod_str;
		debugMsg("from processLinks... m_archive_mod_str: " + m_archive_mod_str);
		for	(_ptr_item = m_file_list.first(); _ptr_item; _ptr_item = m_file_list.next())
		{
			QString str_text = _krawl_inst.readFileContent(_ptr_item->url().path());
			str_text = _krawl_inst.adjustUrl(str_text, m_archive_mod_str, m_mod_str_index);
			_krawl_inst.writeBackContents(_ptr_item->url().path(), str_text);
			
			delete _ptr_item;
			_ptr_item = 0;
		}
		createArchiveIndexFile();
		createTarBall();
	}
//////////////////////////////////////////////////////////////////////////////////////////	
}

void KrawlArchiver::createArchiveIndexFile()
{
	debugMsg("createArchiveIndexFile...");
	emit infoMessage(i18n("Creating archive index file..."));
	QString file_contents_str;
	///////////////////////////////////////////////////////////////////////////////////////
	file_contents_str = "<html>\n";
	file_contents_str += "	<head>\n";
	file_contents_str += "		<script language=\"javascript\">\n";
	file_contents_str += "			function jump2IndexFile()\n";
	file_contents_str += "			{\n";
	file_contents_str += "				document.location.href = \"" + m_index_file_rel_path + "\";\n";
	file_contents_str += "			}\n";
	file_contents_str += "		</script>\n";
	file_contents_str += "	</head>\n";
	file_contents_str += "	<body onload=\"javascript:jump2IndexFile()\">\n";
	file_contents_str += "	</body>\n";
	file_contents_str += "</html>\n";	
	///////////////////////////////////////////////////////////////////////////////////////
	QString file_path = m_tmp_dir->qDir()->path();
	if (file_path.endsWith("/") == false)
		file_path += "/";
	m_archive_index_file_path = file_path + "index.html";
	QFile index_file(m_archive_index_file_path);
	if (index_file.exists() == false && index_file.open(IO_WriteOnly))
	{
		QTextStream stream(&index_file);
		stream<<file_contents_str;
		index_file.close();
	}
}

void KrawlArchiver::createTarBall()
{	
	debugMsg("createTarBall...");
	emit infoMessage(i18n("Creating tarball..."));
	KTar tar_file(m_save_url.path(), "application/x-gzip");
	
	if (tar_file.open(IO_WriteOnly))
	{
		tar_file.addLocalDirectory(m_tmp_url.path(), m_base_dir.dirName());	
		debugMsg("createTarBall...addLocalDirectory ... done");
		tar_file.addLocalFile(m_archive_index_file_path, "index.html");
		debugMsg("createTarBall...addLocalFile ... done");
		tar_file.close();
	}
	else {
		KMessageBox::sorry(parentWidget, i18n("Could not open web-archive for writing"), "Krawlsite");
		debugMsg("couldnt open archive for writing");
	}	
	
	m_tmp_dir->unlink();
	delete m_tmp_dir;
	m_tmp_dir = 0;
		
	emit infoMessage(i18n("Archive successfully created."));
	parentWidget->setCursor(QCursor(Qt::ArrowCursor));
}

void KrawlArchiver::debugMsg(QString _str)
{
	kdDebug(9815578)<<"KrawlArchiver..."<<_str<<endl;
}

KrawlArchiveWidget::KrawlArchiveWidget(QWidget* parent, KURL _base_url, const char* name, WFlags fl) 
							: QWidget(parent, name, fl)
{
    if ( !name )
		setName( "archive_widget" );
	layoutInit();
	m_Archiver = new KrawlArchiver(this, _base_url);
	makeConnections();
}							

KrawlArchiveWidget::~KrawlArchiveWidget()
{
	kdDebug(9815578)<<"KrawlArchiveWidget destructor called."<<endl;
	delete m_Archiver;
	m_Archiver = 0;
}

void KrawlArchiveWidget::layoutInit()
{
    archive_widgetLayout = new QGridLayout( this, 1, 1, 11, 6, "archive_widgetLayout"); 

    m_mesgBoard = new QTextEdit( this, "m_mesgBoard" );
    m_mesgBoard->setReadOnly( TRUE );

    archive_widgetLayout->addWidget( m_mesgBoard, 0, 0 );
    languageChange();
    setMinimumSize(QSize(450, 202));
    clearWState( WState_Polished );
}

void KrawlArchiveWidget::makeConnections()
{
	connect(m_Archiver, SIGNAL(infoMessage(QString )), this, SLOT(slotInfoMesg(QString )));
}

void KrawlArchiveWidget::slotInfoMesg(QString _qstr_mesg)
{
	m_mesgBoard->append(_qstr_mesg + "\n");
}

void KrawlArchiveWidget::languageChange()
{
    setCaption( tr( "Create archive" ) );
}

void KrawlArchiveWidget::startArchiveCreation()
{
	m_Archiver->createArchive();
}
