/***************************************************************************
*   Copyright (C) 2004 by Sayantam Dey                                    *
*   sayantam@gmail.com                                                    *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#ifndef KRAWLARCHIVER_H
#define KRAWLARCHIVER_H

/**
@author Sayantam Dey
*/
#include <kurl.h>
#include <kio/global.h>
#include <kio/jobclasses.h>
#include <kfileitem.h>
#include <ktempdir.h>

#include <qfileinfo.h>
#include <qdir.h>
#include <qwidget.h>

#include "krawler.h"

class KrawlArchiver : public QObject
{
	Q_OBJECT
			
	public:
		KrawlArchiver(QWidget* parent, KURL _base_url);

		~KrawlArchiver();
	
		void createArchive();	
		
	private:
		//attributes
		QWidget* parentWidget;
		KURL m_tmp_url;
		QDir m_base_dir;
		KFileItemList m_file_list;
		QString m_archive_mod_str;
		KURL m_save_url;
		int m_mod_str_index;
		QString m_index_file_rel_path;
		QString m_archive_index_file_path;
		KTempDir* m_tmp_dir;
		bool m_b_dir_list;
		bool m_b_index_file_search;
		bool m_b_index_file_found;
		bool m_b_done_once;
		QString m_index_file_name;
		KURL m_base_dl_dir;
		KIO::ListJob* 	list_job_ptr;
		KIO::CopyJob* copy_job_ptr;
				
		// methods
		void initialize();
		void getFileList(KURL _url);
		void createTarBall();
		void createArchiveIndexFile();
		void searchIndexFile();
		void continueArchival();
		void debugMsg(QString _str);

	protected slots:
		void slotDirListing(KIO::Job *, const KIO::UDSEntryList &);
		void slotCopyJob(KIO::Job *);
		void processLinks(KIO::Job *);
		void slotContinueArchival(KIO::Job *);
	
	signals:
		void infoMessage(QString);
};

class QGridLayout;
class QTextEdit;

class KrawlArchiveWidget : public QWidget
{
	Q_OBJECT
	
	private:
		KrawlArchiver* m_Archiver;
		void layoutInit();
		void makeConnections();			
	public:
		KrawlArchiveWidget(QWidget* parent = 0, KURL _base_url = KURL(), const char* name = 0, WFlags fl = 0);
		~KrawlArchiveWidget();
		
		void startArchiveCreation();
	
	protected:
		QGridLayout* archive_widgetLayout;
		QTextEdit* m_mesgBoard;
	
	protected slots:
		virtual void languageChange();
		void slotInfoMesg(QString);
};
#endif
