/***************************************************************************
 *   Copyright (C) 2004 by Sayantam Dey                                    *
 *   sayantam@gmail.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <kstandarddirs.h>
#include <keditlistbox.h> 
#include <kurlrequester.h>
#include <kglobal.h>
#include <kglobalsettings.h>
#include <kfiledialog.h>
#include <knuminput.h>
#include <kfile.h>
#include <kstaticdeleter.h>
#include <klocale.h>
#include <kdebug.h>
#include <klineedit.h>

#include <qcheckbox.h>
#include <qwidget.h>

#include "krawlconfig.h"

KrawlConfigHandler::KrawlConfigHandler(QWidget* parentWidget):
	KConfigDialog(parentWidget,"Settings",KrawlConfigSkel::self())
{
	crawlPage = new KrawlOptionPage(0,"Crawl Mode");
	urlPage = new UrlOptionsPage(0,"Url Bar"); 
	ldPage = new LocalDirOptionsPage(0,"Local Directory"); 
	fiePage = new FileIncExOptionsPage(0,"File Types"); 
	addPage(crawlPage, i18n("Crawl Mode"), "misc", i18n("Set crawl mode, depth and external link behaviour"));
	addPage(urlPage, i18n("Url Bar"), "enhanced_browsing", i18n("Url location bar settings"));
	addPage(ldPage, i18n("Local Directory"), "folder", i18n("Save options"));
	addPage(fiePage, i18n("File Types"), "filetypes", i18n("File inclusion/exclusion options"));
	loadSavedSettings();
}

KrawlConfigHandler::~KrawlConfigHandler()
{
	kdDebug(9815578)<<"KrawlConfigHandler destructor called."<<endl;	
	delete crawlPage;
	crawlPage = 0;
	delete urlPage;
	urlPage = 0;
	delete ldPage->m_notify_win;
	ldPage->m_notify_win = 0;
	delete ldPage;
	ldPage = 0;
	delete fiePage;
	fiePage = 0;
}

KrawlOptionPage::KrawlOptionPage(QWidget* parent, const char* name, WFlags fl ):
	KrawlOptions(parent, name, fl)
{
}

KrawlOptionPage::~KrawlOptionPage()
{
}

UrlOptionsPage::UrlOptionsPage(QWidget* parent, const char* name, WFlags fl ):
	UrlOptions(parent, name, fl)
{
}

UrlOptionsPage::~UrlOptionsPage()
{
}

LocalDirOptionsPage::LocalDirOptionsPage(QWidget* parent, const char* name, WFlags fl ):
	LocalDirOptions(parent, name, fl)
{
	kcfg_defBaseDirUrl->setMode(KFile::Directory | KFile::ExistingOnly | KFile::LocalOnly);
	connect(kcfg_chkNotifyWin, SIGNAL(clicked()), this, SLOT(cfgNotifyClicked()));
	m_notify_win = 0;
}

LocalDirOptionsPage::~LocalDirOptionsPage()
{
}

void LocalDirOptionsPage::cfgNotifyClicked()
{
	kdDebug(9815578)<<"notifySlot called"<<endl;
	if (kcfg_chkNotifyWin->isChecked())
	{
		if (!m_notify_win)
			m_notify_win = new NotifyWindow(0, 0, false, "Notification Window: Drag to reposition");
		m_notify_win->show();
	}
	else 
	{
		if (m_notify_win)
			m_notify_win->hide();
	}
}

void LocalDirOptionsPage::showEvent(QShowEvent*)
{
	kdDebug(9815578)<<"showEvent called"<<endl;
	cfgNotifyClicked();
}

void LocalDirOptionsPage::hideEvent(QHideEvent*)
{
	kdDebug(9815578)<<"hideEvent called"<<endl;
	if (m_notify_win)
		m_notify_win->hide();
	if (KrawlConfigSkel::showNotifications())
	{
		KrawlConfigSkel::setNotifyWinPos( m_notify_win->pos() );
	}
}

FileIncExOptionsPage::FileIncExOptionsPage(QWidget* parent, const char* name, WFlags fl ):
	FileIncExOptions(parent, name, fl)
{
	m_elbDcExtns->upButton()->setShown(false);
	m_elbDcExtns->downButton()->setShown(false);
	m_elbImageTypes->upButton()->setShown(false);
	m_elbImageTypes->downButton()->setShown(false);
}

FileIncExOptionsPage::~FileIncExOptionsPage()
{
}

void KrawlConfigHandler::slotSetDefaults()
{
	fiePage->m_elbImageTypes->clear();
	fiePage->m_elbDcExtns->clear();
}

void KrawlConfigHandler::slotApplySettings()
{
	KrawlConfigSkel::setFileTypes(fiePage->m_elbImageTypes->items()) ;
	KrawlConfigSkel::setDcFileTypes(fiePage->m_elbDcExtns->items());
}
 
void KrawlConfigHandler::loadSavedSettings()
{
	fiePage->m_elbImageTypes->clear();
	fiePage->m_elbImageTypes->insertStringList(KrawlConfigSkel::fileTypes()); 
	fiePage->m_elbDcExtns->clear();
	fiePage->m_elbDcExtns->insertStringList(KrawlConfigSkel::dcFileTypes());
}

QStringList KrawlConfigHandler::excludedFileExtns()
{
	return fiePage->m_elbImageTypes->items();
}

KPushButton* KrawlConfigHandler::getClearUrlsBtnHandle()
{
	return urlPage->m_btnDelUrlHist;
}

/////////////////////////////////////////////////////////////////////////////
//// skeleton
////////////////////////////////////////////////////////////////////////////


KrawlConfigSkel *KrawlConfigSkel::mSelf = 0;
static KStaticDeleter<KrawlConfigSkel> staticDeleter;

KrawlConfigSkel *KrawlConfigSkel::self()
{
  if ( !mSelf ) {
    staticDeleter.setObject( mSelf, new KrawlConfigSkel() );
    mSelf->readConfig();
  }

  return mSelf;
}

KrawlConfigSkel::KrawlConfigSkel() :
	KConfigSkeleton("libkrawlsitepartrc")
{
	mSelf = this;
	
	setCurrentGroup("Remote location URL bar");
	addItemBool("chkUrlPersist", b_chkUrlPersist, true);
	addItemInt("numMaxUrls", i_numMaxUrls, 500);
	addItemBool("chkDTAllDesks", b_chkDTAllDesks, false);
	addItemBool("chkSelectChecks", b_chkSelectChecks, true);

	setCurrentGroup("Local base directory");
	addItemString("defBaseDirUrl", qstr_defBaseDirUrl, KGlobalSettings::documentPath());
	addItemBool("chkLastBaseDir", b_chkLastBaseDir, true);
	addItemBool("chkAlwaysAskTar", b_always_ask_tar, false);
	addItemBool("chkHiddenFiles", b_showHiddenFiles, false);
	addItemBool("chkNotifyWin", b_showNotifications, true);
		
	setCurrentGroup("Crawling options");
	addItemBool("Crawl", b_Crawl, true);	
	addItemBool("Offline", b_Offline, false);	
	addItemBool("Leech", b_Leech, false);	
	addItemInt("numDefCrawlDepth", i_numDefCrawlDepth, 0);
	addItemBool("chkDefWander", b_chkDefWander, false);	
	addItemBool("chkLinkOutsidePath", b_chkLinkOutsidePath, false);
	
	setCurrentGroup("Excluded file types");
	addItemBool("FileExMode", b_FileExMode, true);
	addItemBool("FileIncMode", b_FileIncMode, false);
}

KrawlConfigSkel::~ KrawlConfigSkel()
{
  kdDebug(9815578)<<"KrawlConfigSkel destructor called"<<endl;
  if ( mSelf == this )
  	staticDeleter.setObject( mSelf, 0, false );
}

QStringList KrawlConfigSkel::fileTypes()
{
	mSelf->config()->setGroup("Excluded file types");
	return mSelf->config()->readListEntry("ExcludedTypes", ','); 
}

void KrawlConfigSkel::setFileTypes(QStringList _fileTypes)
{
	mSelf->config()->setGroup("Excluded file types");
	mSelf->config()->writeEntry("ExcludedTypes", _fileTypes);
	mSelf->writeConfig();
}		

QString KrawlConfigSkel::lastLocalDirectory()
{
	mSelf->config()->setGroup("Local base directory");
	return mSelf->config()->readPathEntry("LastBaseDlPath", "");
}

void KrawlConfigSkel::setLastLocalDirectory(QString _local_dir_path_str)
{
	mSelf->config()->setGroup("Local base directory");
	mSelf->config()->writeEntry("LastBaseDlPath", _local_dir_path_str);
	mSelf->writeConfig();
}

QPoint KrawlConfigSkel::notifyWinPos()
{
	mSelf->config()->setGroup("Local base directory");
	return mSelf->config()->readPointEntry("NotifyWindowPos", new QPoint(-1, -1));
}

void KrawlConfigSkel::setNotifyWinPos(QPoint _p)
{
	mSelf->config()->setGroup("Local base directory");	
	mSelf->config()->writeEntry("NotifyWindowPos", _p);
	mSelf->writeConfig();
}

QStringList KrawlConfigSkel::savedUrls()
{
	mSelf->config()->setGroup("Remote location URL bar");
	return mSelf->config()->readListEntry("SavedUrls",',') ;
}

void KrawlConfigSkel::setSavedUrls(QStringList _url_str_list)
{
	mSelf->config()->setGroup("Remote location URL bar");
	mSelf->config()->writeEntry("SavedUrls", _url_str_list);
	mSelf->writeConfig();
}

void KrawlConfigSkel::setDcFileTypes(QStringList _dc_file_list)
{
	mSelf->config()->setGroup("Dynamic Content File Types");
	mSelf->config()->writeEntry("DcFileTypes", _dc_file_list);
	mSelf->writeConfig();
}

QStringList KrawlConfigSkel::dcFileTypes()
{
	mSelf->config()->setGroup("Dynamic Content File Types");
	return mSelf->config()->readListEntry("DcFileTypes", ','); 
}

/*$SPECIALIZATION$*/


#include "krawlconfig.moc"

