/***************************************************************************
 *   Copyright (C) 2004 by Sayantam Dey                                    *
 *   sayantam@gmail.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef KRAWLCONFIG_H
#define KRAWLCONFIG_H

#include <kconfig.h>
#include <kconfigskeleton.h>
#include <kconfigdialog.h>
#include <kpushbutton.h>
#include <keditlistbox.h>

#include "krawloptions.h"
#include "urloptions.h"
#include "localdiroptions.h"
#include "fileincexoptions.h"
#include "notifiywindow.h"

///////////////////////////////////////////////////////////////////////////////////
/// widget subclasses
///////////////////////////////////////////////////////////////////////////////////
class KrawlOptionPage;
class UrlOptionsPage;
class LocalDirOptionsPage;
class FileIncExOptionsPage;

class KrawlConfigHandler : public KConfigDialog
{
	Q_OBJECT
			
	public:
	KrawlConfigHandler(QWidget* parentWidget);
	~KrawlConfigHandler();
	/*$PUBLIC_FUNCTIONS$*/
	KPushButton* getClearUrlsBtnHandle();
	QStringList excludedFileExtns();	
	
	KrawlOptionPage *crawlPage;
	UrlOptionsPage *urlPage;
	LocalDirOptionsPage *ldPage;
	FileIncExOptionsPage *fiePage;
	
	protected:
	void loadSavedSettings();
	
	public slots:
	/*$PUBLIC_SLOTS$*/
	void slotApplySettings();
	void slotSetDefaults();
};


class KrawlOptionPage : public KrawlOptions
{
	Q_OBJECT 
			
	public:
		
	KrawlOptionPage( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
	~KrawlOptionPage();
};

class UrlOptionsPage : public UrlOptions
{
	Q_OBJECT
			
	public:
		
	UrlOptionsPage( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
	~UrlOptionsPage();
};

class LocalDirOptionsPage : public LocalDirOptions
{
	Q_OBJECT
			
	public:
		
	LocalDirOptionsPage( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
	~LocalDirOptionsPage();
	
	NotifyWindow* m_notify_win;
	
	protected slots:
		void cfgNotifyClicked();
		void showEvent(QShowEvent*);
		void hideEvent(QHideEvent*);	
};

class FileIncExOptionsPage : public FileIncExOptions
{
	Q_OBJECT
			
	public:
		
	FileIncExOptionsPage( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
	~FileIncExOptionsPage();
};

 class KrawlConfigSkel : public KConfigSkeleton
{
	public:
		~KrawlConfigSkel();
		static KrawlConfigSkel* self();
		
		static void writeConfig()
		{
			static_cast<KConfigSkeleton*>(self())->writeConfig();
		}
		static QString defaultBaseDir()
		{
			return self()->qstr_defBaseDirUrl;
		}
		static bool chkLastBaseDirUrl()
		{
			return self()->b_chkLastBaseDir;
		}
		static bool chkAlwaysAskTarIndex()
		{
			return self()->b_always_ask_tar;
		}
		static bool showHiddenFiles()
		{
			return self()->b_showHiddenFiles;
		}
		static bool chkUrlPersistance()
		{
			return self()->b_chkUrlPersist;
		}
		static int maxNumUrls()
		{
			return self()->i_numMaxUrls;
		}
		static bool chkDTAllDesks()
		{
			return self()->b_chkDTAllDesks;
		}
		static bool chkSelectChecks()
		{
			return self()->b_chkSelectChecks;
		}
		static bool showNotifications()
		{
			return self()->b_showNotifications;
		}
		static bool fileExMode()
		{
			return self()->b_FileExMode;
		}
		static bool fileIncMode()
		{
			return self()->b_FileIncMode;
		}
		static bool simpleCrawl()
		{
			return self()->b_Crawl;
		}
		static bool offlineBrowse()
		{
			return self()->b_Offline;
		}
		static bool leech()
		{
			return self()->b_Leech;
		}
		static int defCrawlDepth()
		{
			return self()->i_numDefCrawlDepth;
		}
		static bool chkOutsideDomain()
		{
			return self()->b_chkDefWander;
		}
		static bool chkOutsidePath()
		{
			return self()->b_chkLinkOutsidePath;
		}
		
		static QString lastLocalDirectory();
		static void setLastLocalDirectory(QString _local_dir_path_str);
		static QPoint notifyWinPos();
		static void setNotifyWinPos(QPoint _p);
		static QStringList fileTypes();
		static void setFileTypes(QStringList _fileList);
		static QStringList savedUrls();
		static void setSavedUrls(QStringList _url_str_list);
		static QStringList dcFileTypes();
		static void setDcFileTypes(QStringList _dc_file_list);
		
	protected:
		KrawlConfigSkel();
		static KrawlConfigSkel *mSelf;
		
		QString qstr_defBaseDirUrl;
		bool b_chkLastBaseDir;
		bool b_always_ask_tar;
		bool b_showHiddenFiles;
		bool b_showNotifications;
		
		bool b_chkUrlPersist;
		int i_numMaxUrls;
		bool b_chkDTAllDesks;
		bool b_chkSelectChecks;
		
		bool b_FileExMode;
		bool b_FileIncMode;
		
		bool b_Crawl;
		bool b_Offline;
		bool b_Leech;
		int i_numDefCrawlDepth;
		bool b_chkDefWander;
		bool b_chkLinkOutsidePath;
};

#endif

