/***************************************************************************
*   Copyright (C) 2004 by Sayantam Dey                                    *
*   sayantam@gmail.com                                                    *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include <kdebug.h>
#include <kmimetype.h>
#include <kio/jobclasses.h>
#include <kio/scheduler.h>
#include <krun.h>
#include <klocale.h>

#include <qfile.h>
#include <qtextstream.h>
#include <qdir.h>
#include <qregexp.h>
#include <qapplication.h>

#include "krawlerevent.h"
#include "krawler.h"

Krawler::Krawler() : QObject(), QThread()
{ }

Krawler::Krawler(QString _str_url) : QObject(), QThread()
{
	m_str_url = _str_url;
}

Krawler::Krawler(QObject* _object, QWidget* _widget, t_LeechJobList _checked_urls, QString _qstr_init_dir) : QObject(), QThread()
{
	m_leech_job = true;
	m_checked_Urls = _checked_urls;
	m_object = _object;
	init(_widget, _qstr_init_dir);
}

Krawler::Krawler(QObject* _object, QWidget* _widget, QString _qstr_init_dir, t_KrawlJobList _job_list) 
			: QObject(), QThread()
{
	m_job_list = _job_list;
	m_object = _object;
	init(_widget, _qstr_init_dir);
}

Krawler::~Krawler()
{
	debugMsg("krawler destructor called");
}
	
void Krawler::init(QWidget* _widget, QString _qstr_init_dir)
{
	m_widget = _widget;
	m_qstr_init_dir = _qstr_init_dir;
	jumpToNextLink = false;
	jumpToNextUrl = false;
	paused = false;

}

void Krawler::setJobParameters(JobOptions _job_opt)
{
	m_iMaxLevel = _job_opt.crawl_depth;
	m_bTraverseOut = _job_opt.traverseOut;
	m_bLinkOutsidePath = _job_opt.linkOutsidePath;
	m_qstr_list_ExcludedFileTypes = _job_opt.fileTypeList;
	m_iKrawlMode = _job_opt.crawl_mode;
	m_fileType_only = (! _job_opt.exclusive_flag);
	m_dcFileTypes = _job_opt.dcTypes;
}

void Krawler::run()
{
	int result_code = -1;
	m_stop_krawling = false;
	if (m_checked_Urls.size() > 0)
	{
		debugMsg("start leeching");    
		t_LeechJobList::iterator it;
		int dl_result_code = -1;
		QString dummy;
		int leech_job_index = 1;
		for (it = m_checked_Urls.begin(); it != m_checked_Urls.end(); it++) 
		{
			if (checkAndPause())
				it--;
			t_LeechJob leech_job = (*it);
			m_index = leech_job.m_index;
			QValueList<t_LeechUrl>::iterator leech_it;
			int index = 1;
			for ( leech_it = leech_job.m_LeechUrlList.begin(); leech_it != leech_job.m_LeechUrlList.end(); leech_it++ )
			{
				t_LeechUrl leech_url = (*leech_it);
				downloadFile(leech_url.m_str_url, dl_result_code, dummy, leech_url.m_index, leech_url.m_url_type);
// 				emit sigSetImgStats(index++, (int)leech_job.m_LeechUrlList.size());
				post_event(new KwImgStatsEvent(index++, (int)leech_job.m_LeechUrlList.size()));
				if (dl_result_code == C_KILL)
					break;
			}
			if (dl_result_code == C_KILL)
				break;
// 			emit sigSetLinkStats( leech_job_index++, (int)m_checked_Urls.size() ); 
			post_event(new KwLinkStatsEvent(leech_job_index++, (int)m_checked_Urls.size()));
		}
// 		emit sigSetStatusText("Performing clean up...");
		post_event(new KwStatusTextEvent("Performing clean up..."));
		performCleanup();
// 		emit downloadedAllSelected();
		post_event(new KwSelectedDlCompleteEvent());
		debugMsg("done leeching");    
	}
	else 
	{
		t_KrawlJobList::iterator it = m_job_list.begin();
		int index = 0;
		t_KrawlJob krawl_job;
		while (it != m_job_list.end())
		{
			krawl_job = (t_KrawlJob)(*it);
			m_url = krawl_job.m_url_str;
			JobOptions job_opt = krawl_job.job_options;
			debugMsg(job_opt.allAttributes());
			setJobParameters(job_opt);
			m_url.cleanPath();
			m_lock.lock();
			jumpToNextUrl = false;
			m_lock.unlock();
			if (checkAndPause())
				it--;
			m_index = index;
// 			emit nextJobIndex(index++, m_url);
			post_event(new KwNextJobEvent(index++, m_url));
			if ((result_code = singleCrawlJob()) == C_KILL)
				break;
			it++;
		}
	}
// 	emit finished(result_code);
	post_event(new KwFinishedEvent(result_code));
}	

int Krawler::singleCrawlJob()
{
	debugMsg("start krawling" + m_url.url());    
	KURL::List lVisitedUrlList;	
	// crawl, crawl
	int result_code = startCrawling(m_url, 0, lVisitedUrlList);
// 	emit sigSetStatusText("Performing clean up...");
	post_event(new KwStatusTextEvent(i18n("Performing clean up...")));
	performCleanup();
	debugMsg("done krawling");    
	return result_code;
}

/////////////////////////////////////////
// SLOT
void Krawler::stopCrawlingClicked()
{
	m_lock.lock();
	if (m_download_in_progress)
	{
		m_current_kio_job->kill(true);
		m_bad_download = true;
		m_download_in_progress = false;
		debugMsg("active download killed");
	}
	m_stop_krawling = true;
	m_lock.unlock();
}

void Krawler::slotResult(KIO::Job* kio_job)
{
	int result_code = kio_job->error();
	if (result_code != 0)
	{
		QString _err_mesg = kio_job->errorString();
		m_bad_download = true;
// 		emit sigErrorMesg(_err_mesg);
		post_event(new KwErrorMesgEvent(_err_mesg));
	}
	m_lock.lock();
	m_download_in_progress = false;
	m_lock.unlock();
}

void Krawler::slotProcessedSize(KIO::Job*, KIO::filesize_t file_size)
{
	m_ul_last_processed_job_size = m_ul_processed_job_size;
	m_ul_processed_job_size = KIO::number(file_size).toULong();
// 	emit sigProcessedSize(m_ul_processed_job_size - m_ul_last_processed_job_size);
	post_event(new KwProcessedEvent(m_ul_processed_job_size - m_ul_last_processed_job_size));
	
}

void Krawler::gotoNextLink()
{
// 	QMutex mutex;
	m_lock.lock();
	if (m_download_in_progress)
	{
		m_current_kio_job->kill(true);
		m_bad_download = true;
		m_download_in_progress = false;
	}
	jumpToNextLink = true;
	m_lock.unlock();
}

void Krawler::gotoNextURL()
{
// 	QMutex mutex;
	m_lock.lock();
	if (m_download_in_progress)
	{
		m_current_kio_job->kill(true);
		m_bad_download = true;
		m_download_in_progress = false;
	}
	jumpToNextUrl = true;
	m_lock.unlock();
}

void Krawler::pauseKrawl(bool _flag)
{
// 	QMutex mutex;
	m_lock.lock();
	if (m_download_in_progress)
	{
		m_current_kio_job->kill(true);
		m_bad_download = true;
		m_download_in_progress = false;
	}
	paused = _flag;
	m_lock.unlock();
}

void Krawler::slotIgnoreLink(const QString& _url_str)
{
	m_ig_mutex.lock();
	QStringList ig_url_list = QStringList::split(",", _url_str, false);
	if (ig_url_list.count() <= 1)
	{
		m_ignore_urls.append(_url_str);
	}
	else
	{
		for ( QStringList::Iterator ite = ig_url_list.begin(); ite != ig_url_list.end(); ite++)
			m_ignore_urls.append(*ite);
	}
	m_ig_mutex.unlock();
	if (_url_str == m_url.url())
		gotoNextLink();
}

void Krawler::slotPercent(KIO::Job *_job, unsigned long _data)
{
	post_event(new KwPercentEvent(_job, _data));
}

void Krawler::slotSpeed(KIO::Job *_job, unsigned long _data)
{
	post_event(new KwSpeedEvent(_job, _data));
}

void Krawler::slotInfoMessage(KIO::Job *_job, const QString& _str)
{
	post_event(new KwStatusTextEvent(_job, _str)); 
}

///////////////////////////////////////////

bool Krawler::continueKrawling()
{
	m_lock.lock();
	if (m_stop_krawling) 
	{
		m_lock.unlock();
		debugMsg("krawler has been stopped");			
		return false;
	}
	m_lock.unlock();
	return true;	
}

bool Krawler::checkAndPause()
{
// 	QMutex mutex;
	m_lock.lock();
	if (!paused)
	{
		m_lock.unlock();
		return false;
	}
	m_lock.unlock();
	// else wait
	while (1)
	{
		m_lock.lock();
		if ( !paused )
			break;
		m_lock.unlock();
		sleep(1);
	}
	m_lock.unlock();
	return true;
}

void Krawler::performCleanup()
{
	TempFileList::iterator it;
	for (it = temp_file_list.begin(); it != temp_file_list.end(); it++)
	{
		KTempFile file_item = KTempFile (*it);
		file_item.unlink();
	}
}

bool Krawler::ignoreLink(QString _url_str)
{
	m_ig_mutex.lock();
	bool ignore = m_ignore_urls.contains(_url_str) > 0 ? true : false;
	m_ig_mutex.unlock();
	return ignore;
}
	
QString Krawler::downloadFile(KURL embedUrl, int& result_code, QString& _embed_dir_str, uint _index, int _url_type)
{
	// check if user has hit stop
	if (continueKrawling() == false)
	{
 		result_code = DL_KILL;
		return 0;
	}
	
	// check if file is in excluded list
	if (isFileTypeExcluded(embedUrl.url()))
	{
		result_code = DL_SKIP;
// 		debugMsg(embedUrl.url() + " isFileTypeExcluded returned true");
		return 0;
	}
	
	// we need to check the url and create the necessary directories if the url has a trailing /
	QString dir_str = embedUrl.directory(false,false);
	
	//clean up the directory string
	dir_str = dir_str.simplifyWhiteSpace();
	for (uint j = 0; j < dir_str.length(); j++)
	{
		QChar qch(dir_str[j]);
		if (qch.isLetterOrNumber() ||	qch == '/' || qch == '_' || qch == '.')
		{
			continue;
		}
		else {
			dir_str.remove(dir_str[j]);
			j--;
		}
	}
	QStringList dirs_list = QStringList::split("/", dir_str);
	dir_str = m_qstr_init_dir + "/" + dir_str;
	
	// create directory structure
	QDir _dir(dir_str);
	// remove special characters
	dir_str = _dir.absPath();
	_embed_dir_str = dir_str;
	if (_embed_dir_str.endsWith("/") == false)
		_embed_dir_str += "/";
	
	if (_dir.exists() == false)
	{
// 		KRun::runCommand("mkdir -p " + dir_str);
// 		_dir.mkdir(dir_str);
		QDir q_init_dir(m_qstr_init_dir);
		QString cr_dir_str = ""; 
		for(QStringList::Iterator it = dirs_list.begin(); it != dirs_list.end(); ++it)
		{
			cr_dir_str += ( (*it) + "/" );
			q_init_dir.mkdir(cr_dir_str);
		}
	}
	
	QString fileName;
	
	fileName = embedUrl.fileName(false);
	
	bool no_file = false;
	if (fileName == 0 || fileName.isEmpty())
	{
		if (m_url == embedUrl && m_iKrawlMode == Offline_Browser) // this is the start url
		{
			fileName = "index.html";
		}
		else
		{ 
			no_file = true;
			KTempFile tempFile;
			tempFile.close();
			fileName = tempFile.name();
			temp_file_list.append(tempFile);
		}
	}
	bool file_exists = false;
	if (no_file == false) 
	{
		fileName = dir_str + "/" + fileName;
		if (QFile(fileName).exists() && !isDynamicContent(fileName))
		{
			result_code = DL_SKIP;
// 			debugMsg("File exists ....file name: " + fileName);
			file_exists = true;
		}
	}
	
	if (file_exists == false)
	{
// 		emit sigCurrentDlUrl(embedUrl.url(), dl_url_type);
		post_event(new KwCurrentDlUrlEvent(embedUrl.url(), dl_url_type));
// 		emit sigMarker((uint)m_index, _index, _url_type );
		post_event(new KwMarkerEvent((uint)m_index, _index, _url_type));
		
		m_ul_processed_job_size = 0;
		m_lock.lock();
		m_download_in_progress = true; // 
		m_bad_download = false;
		KIO::Scheduler::checkSlaveOnHold(true);	
		m_current_kio_job = KIO::file_copy(embedUrl, KURL(fileName), -1, true, false, false);
		
		m_current_kio_job->setWindow(m_widget);
		
		connect(m_current_kio_job, SIGNAL(result(KIO::Job *)), 
				this, SLOT(slotResult(KIO::Job *)));
		
		connect(m_current_kio_job, SIGNAL(percent(KIO::Job *, unsigned long)), 
				SLOT(slotPercent(KIO::Job *, unsigned long)));
		
		connect(m_current_kio_job, SIGNAL(processedSize(KIO::Job *, KIO::filesize_t)),
				this, SLOT(slotProcessedSize(KIO::Job*, KIO::filesize_t )));
		
		connect(m_current_kio_job, SIGNAL(speed(KIO::Job *, unsigned long )),
				SLOT(slotSpeed(KIO::Job *, unsigned long )));
		
		connect(m_current_kio_job, SIGNAL(infoMessage(KIO::Job *, const QString &)),
				SLOT(slotInfoMessage(KIO::Job *, const QString &)));
		
		m_lock.unlock();

		// wait till this is compeleted
		while (1)
		{
			m_lock.lock();
			if (m_download_in_progress == false) 
			{
				m_lock.unlock();
				break;
			}
			m_lock.unlock();
			sleep(1);
		}
		
		m_lock.lock();
		if (m_stop_krawling)
		{
			result_code = DL_KILL;
			m_lock.unlock();
			return 0;
		}
		m_lock.unlock();
		
		m_lock.lock();
		if (jumpToNextLink)
		{
			result_code = DL_SKIP;
			debugMsg("jumpToNextLink is true... skipping this link");
			jumpToNextLink = false;
			m_lock.unlock();
			return 0;
		}
		m_lock.unlock();
		
		m_lock.lock();
		if (jumpToNextUrl)
		{
			result_code = DL_SKIP;
			debugMsg("jumpToNextUrl is true... skipping this link");
			m_lock.unlock();
			return 0;
		}
		m_lock.unlock();
		
		m_lock.lock();
		if (paused)
		{
			result_code = DL_SKIP;
			debugMsg("pause is true... skipping this link");
			m_lock.unlock();
			return 0;
		}
		m_lock.unlock();
		
		m_lock.lock();
		if (m_bad_download)
		{
			result_code = DL_FAIL;
			m_lock.unlock();
			return 0;
		}
		m_lock.unlock();
		
		if (no_file) 
		{
			result_code = DL_DIR;
		}
		else result_code = DL_OK;
	}
	
// 	emit downloadedFile(result_code, embedUrl.url(), fileName);
	post_event(new KwFileDlComplete(result_code, embedUrl.url(), fileName));
	return fileName;
}


int Krawler::startCrawling(KURL embedUrl, int level, KURL::List lVisitedUrlList, QString _parent_file_name, uint _index, int _url_type)
{
	debugMsg("start krawling" + embedUrl.url()); 
	if (ignoreLink(embedUrl.url()))
		return C_SKIP;
	
	int dl_result_code = -1;
	QString dl_file_name;
	QString htmlRef = ""; 
	QString url_dir_str = QString::null;
	KURL orig_embed_url = embedUrl;
	if (embedUrl.hasHTMLRef())
	{
		htmlRef = embedUrl.htmlRef();
		QString full_url_str = embedUrl.url();
		QString link_url_str = full_url_str.section('#', 0, 0);
		embedUrl = KURL::KURL(link_url_str);
	}
	if (embedUrl.isLocalFile() == false) 
	{
		dl_file_name = downloadFile(embedUrl, dl_result_code, url_dir_str, _index, _url_type);
// 		debugMsg(QString("download result code: %1").arg(dl_result_code));
	}
	else dl_file_name = embedUrl.url();

	// check if user has hit stop - second check, because if a download job was in progress, killing it, returns control
	// to this point..i guess  :-}
	if (continueKrawling() == false) 
		return C_KILL;
	
	m_lock.lock();
	if (jumpToNextUrl)
	{
		m_lock.unlock();
		return C_SKIP;
	}
	m_lock.unlock();
	
	if (dl_file_name == 0)
		return C_FAIL;
	
	// if we are offline browser, replace the url with the filename in the parent file
	if (m_iKrawlMode == Offline_Browser)
	{
		QString _tmp_str_holder = dl_file_name;
		
		if (dl_result_code == DL_DIR)
			_tmp_str_holder = url_dir_str;
		if (htmlRef != "")
			_tmp_str_holder += ("#"+htmlRef);
		
		replaceUrl(_parent_file_name, _tmp_str_holder, orig_embed_url.url());
	}
	
	if (embedUrl.isLocalFile())
	{
// 		debugMsg("skipping coz its local file: " + embedUrl.url());
		return C_SKIP;
	}
	
	if (level > m_iMaxLevel)
		return C_GT_MAX_LEVEL; // no more crawling, go back :)
	
	// only if the d/l'ed file was of type text/html do we go further
	KMimeType::Ptr mime_type = KMimeType::findByFileContent(dl_file_name);
	
	if (mime_type->name().compare("text/html") != 0) 
		return C_NOT_HTML;
	
	if (lVisitedUrlList.findIndex( embedUrl ) != -1)
	{
// 		debugMsg("skipping coz its already crawled" + embedUrl.url());
		return C_SKIP;
	}
	
	// unless we are asked to, we will not parse external links with mime type text/html, just download
	if (krawlLink(embedUrl) == false)
	{
// 		debugMsg("skipping coz its an external link:" + embedUrl.url());
		return C_SKIP;
	}

// 	emit sigSetStatusText(i18n("Parsing " + embedUrl.url()));
	post_event(new KwStatusTextEvent(i18n("Parsing " + embedUrl.url())));

    QString str = readFileContent(dl_file_name);
	
	if (str == 0)
		return C_FAIL;
	
	// parse html
	QStringList::Iterator it;
	int c_result_code = -1;
	int index = 1;
	
	//frameset
	QStringList frame_url_list = frameList(str);
	for (it = frame_url_list.begin(); it != frame_url_list.end(); it++)
	{
		KURL parsedLink = KURL(*it);
// 		post_event(new KwLinkStatsEvent(index, (int)links_list.size()));
			
		m_lock.lock();
		if (jumpToNextUrl)
		{
			c_result_code = C_SKIP;
			m_lock.unlock();
			break;
		}
		m_lock.unlock();
			
		c_result_code = startCrawling(parsedLink, level + 1, lVisitedUrlList, dl_file_name, (uint)index++, HREF);
			
		if (checkAndPause())
			it--;
// 			debugMsg(QString("href crawl result code=%1").arg(c_result_code));
		if (c_result_code == C_KILL)
			break;
	}
	
	if (c_result_code == C_KILL || c_result_code == C_SKIP)
		return c_result_code;	
	
// 	emit sigSetStatusText(i18n("Parsing file for image tags..."));
	post_event(new KwStatusTextEvent(i18n("Parsing file for image tags...")));
	QStringList imgs_list = imgSrcList(str, embedUrl);
	debugMsg(QString("size of imgs_list=%1").arg(imgs_list.size()));
	
// 	emit sigSetStatusText(i18n("Parsing file for anchor tags..."));
	post_event(new KwStatusTextEvent(i18n("Parsing file for anchor tags...")));
	QStringList links_list = hrefList(str, embedUrl);
	debugMsg(QString("size of links_list=%1").arg(links_list.size()));
// 	emit sigParsedLink(m_index, embedUrl.url(), level, links_list, imgs_list);
	post_event(new KwParsedLinkEvent(m_index, (const QString&)embedUrl.url(), level, links_list, imgs_list));
	debugMsg("supposed to add the url info here");
	if (m_iKrawlMode == Leech)
	{
// 		emit leechedLink();
		post_event(new KwLeechedLinkEvent());
		return C_OK;	
	}
	if (m_iKrawlMode == Offline_Browser)
		writeBackContents(dl_file_name, str);
	
	// add the embed url to the crawled list of urls
	lVisitedUrlList.append(embedUrl);
	
	dl_url_type = IMG;	
	for (it = imgs_list.begin(); it != imgs_list.end(); it++) 
	{
		KURL downloadLink = KURL(*it);
// 		emit sigSetImgStats(index, (int)imgs_list.size());
		post_event(new KwImgStatsEvent(index, (int)imgs_list.size()));
// 		emit sigMarker((uint)m_index, (uint)index++, IMG );
// 		post_event(new KwMarkerEvent((uint)m_index, (uint)index++, IMG));
		
		m_lock.lock();
		if (jumpToNextUrl)
		{
			c_result_code = C_SKIP;
			m_lock.unlock();
			break;
		}
		m_lock.unlock();
		
		c_result_code = startCrawling(downloadLink, level + 1, lVisitedUrlList, dl_file_name, (uint)index++, IMG);
		if (checkAndPause())
			it--;
		if (c_result_code == C_KILL)
			break;
	}

	index = 1;
	dl_url_type = HREF;
	if (c_result_code != C_KILL && c_result_code != C_SKIP) 
	{
		for (it = links_list.begin(); it != links_list.end(); it++)
		{
			KURL parsedLink = KURL(*it);
// 			emit sigSetLinkStats(index, (int)links_list.size());
			post_event(new KwLinkStatsEvent(index, (int)links_list.size()));
// 			emit sigMarker((uint)m_index, (uint)index++, HREF );
// 			post_event(new KwMarkerEvent((uint)m_index, (uint)index++, HREF));
			
			m_lock.lock();
			if (jumpToNextUrl)
			{
				c_result_code = C_SKIP;
				m_lock.unlock();
				break;
			}
			m_lock.unlock();
			
			c_result_code = startCrawling(parsedLink, level + 1, lVisitedUrlList, dl_file_name, (uint)index++, HREF);
			
			if (checkAndPause())
				it--;
// 			debugMsg(QString("href crawl result code=%1").arg(c_result_code));
			if (c_result_code == C_KILL)
				break;
		}
	}	
	
	return c_result_code;
}

QStringList Krawler::imgSrcList(QString& str_text, const KURL embedUrl, bool decorateUrl)
{
// 	debugMsg("imgSrcList called...");
	QStringList q_str_list;
	
	int beg_index = 0, end_index = 0;
	QString search_begin_tag = "img";
	QString search_end_tag = ">";
	QString embedUrl_proto, embedUrl_host, embedUrl_dir;
	kurlAttributes(embedUrl, embedUrl_proto, embedUrl_host, embedUrl_dir);
	while ((beg_index = str_text.find(search_begin_tag, beg_index, false)) != -1)
	{
		end_index = str_text.find(search_end_tag, beg_index, false);
		
		// string between img and closing > tag. most likely composition: src="link" or src='link' with name=id thrown in perhaps :)
		QString q_tmp_str = substring(str_text, beg_index + search_begin_tag.length(), end_index);
		
		// search for src
		int src_tag_index = q_tmp_str.find("src",0,false);
		
		if (src_tag_index == -1)
		{
			beg_index = end_index + search_end_tag.length();
			continue;
		}
		
		QString q_tmp_str_storage = q_tmp_str;
		QString q_str_link = stringWithinQuotes(embedUrl_proto, embedUrl_host, embedUrl_dir, q_tmp_str, src_tag_index, decorateUrl);
		
// 		debugMsg("*Parsed img links:");	
		if (q_str_link.length() != 0)
		{
			q_str_list.append(q_str_link);
		}
		
		// replace q_tmp_str_storage with q_tmp_str in str_text
		str_text.replace(q_tmp_str_storage, q_tmp_str);
		
		beg_index = end_index + search_end_tag.length() + (q_tmp_str.length() - q_tmp_str_storage.length());
		if (continueKrawling() == false)
			break;
		
		m_lock.lock();
		if (jumpToNextLink)
		{
			jumpToNextLink = false;
			m_lock.unlock();
			break;
		}
		m_lock.unlock();
		
		m_lock.lock();
		if (jumpToNextUrl)
		{
			m_lock.unlock();
			break;
		}
		m_lock.unlock();
		
		checkAndPause();
	}
	
	return q_str_list;	
}


QStringList Krawler::hrefList(QString& str_text, const KURL embedUrl, bool decorateUrl)
{
//     debugMsg("hrefList called...");
	QStringList q_str_list;
	
	int beg_index = 0, end_index = 0;
	QString search_begin_tag = "href";
	QRegExp regExp("\\bhref\\s*=");
	regExp.setCaseSensitive(false);
	QString search_end_tag = ">";
	QString embedUrl_proto, embedUrl_host, embedUrl_dir;
	kurlAttributes(embedUrl, embedUrl_proto, embedUrl_host, embedUrl_dir);
	while ((beg_index = str_text.find(regExp, beg_index)) != -1)
	{
// 		beg_index = str_text.find(search_begin_tag, beg_index, false);
		end_index = str_text.find(search_end_tag, beg_index, false);
		
		// string between href and closing > tag. most likely composition: ="link" or ='link'
		QString q_tmp_str = substring(str_text, beg_index + search_begin_tag.length(), end_index);
		QString q_tmp_str_storage = q_tmp_str;
		QString q_str_link = stringWithinQuotes(embedUrl_proto, embedUrl_host, embedUrl_dir, q_tmp_str, 0, decorateUrl);
// 		debugMsg("#Parsed href links:");			
		if (q_str_link.length() != 0)
		{
			q_str_list.append(q_str_link);
		}
		// replace q_tmp_str_storage with q_tmp_str in str_text
		str_text.replace(q_tmp_str_storage, q_tmp_str);
		beg_index = end_index + search_end_tag.length() + (q_tmp_str.length() - q_tmp_str_storage.length());
		if (continueKrawling() == false)
			break;
		
		m_lock.lock();
		if (jumpToNextLink)
		{
			jumpToNextLink = false;
			m_lock.unlock();
			break;
		}
		m_lock.unlock();
		
		m_lock.lock();
		if (jumpToNextUrl)
		{
			m_lock.unlock();
			break;
		}
		m_lock.unlock();
		
		checkAndPause();
	}
	return q_str_list;	
}

QStringList Krawler::frameList(const QString& str_text)
{
	QRegExp re("<FRAME.+SRC=(?:\"|')?([^\"'\\s]+)(?:\"|')?\\s*.+>");
	re.setCaseSensitive(false);
	re.setMinimal(true);
	int pos = -1;
	QStringList frame_url_list;
	while((pos = str_text.find(re, pos+1)) != -1)
	{
		frame_url_list.append(re.cap(1));
	}
	return frame_url_list;
}

QString Krawler::stringWithinQuotes(const QString _parent_url_proto, const QString _parent_url_host, const QString _parent_url_dir, QString& quoted_text, const int begin_index, bool decorateUrl)
{
// 	debugMsg("stringWithinQuotes called... begin_index=");
	QString q_str_link;
	
	// search for "
	int beg_dbl_quote_index = quoted_text.find("\"", begin_index,false);
	
	if (beg_dbl_quote_index != -1) // match found
	{
		// get the ending "
		int end_dbl_quote_index = quoted_text.find("\"", beg_dbl_quote_index+1, false);
		if (end_dbl_quote_index == -1)
		{
			// god help us.. some ppl cant write html @-)
// 			emit sigErrorMesg("Malformed html: No closing \" found on href tag");
			post_event(new KwErrorMesgEvent(i18n("Malformed html: No closing \" found on href tag")));
			debugMsg("Malformed html: No closing \" found on href tag");
		}
		else {
			q_str_link = substring(quoted_text, beg_dbl_quote_index+1, end_dbl_quote_index);
		}
	}
	else // check for ' 
	{
		int beg_single_quot_index = quoted_text.find("\'",0,false);
		if (beg_single_quot_index != -1) // match found
		{
			// get the closing '
			int end_single_quot_index = quoted_text.find("\'", beg_single_quot_index+1, false);
			if (end_single_quot_index == -1)
			{
				//which dufus wrote this ?! 
// 				emit sigErrorMesg("Malformed html: No closing \' found on href tag");
				post_event(new KwErrorMesgEvent(i18n("Malformed html: No closing \' found on href tag")));
				debugMsg("Malformed html: No closing \' found on href tag");
			}
			else {
				q_str_link = substring(quoted_text, beg_single_quot_index+1, end_single_quot_index);
			}
		}
	}
	if (q_str_link.length() == 0) //thanks to the dufus who forgot " or '
	{
		// get = token.. 
		int equal_token_index = quoted_text.find("=", begin_index, false);
		q_str_link = substring(quoted_text, equal_token_index+1);
		q_str_link = q_str_link.simplifyWhiteSpace();
		q_str_link = q_str_link.stripWhiteSpace();
		q_str_link = q_str_link.section(' ', 0, 0, QString::SectionSkipEmpty);
	}
	// clear whitespace at front and back
	q_str_link = q_str_link.stripWhiteSpace();
// 	debugMsg("stringWithinQuotes... q_str_link before decoration:");
	if (decorateUrl)
	{
		QString q_str_link_storage = q_str_link;
		q_str_link = createUrlStrfromPath(_parent_url_proto, _parent_url_host, _parent_url_dir, q_str_link);
		
		// replace occurrence of q_str_link_storage with q_str_link in quoted_text
		quoted_text.replace(q_str_link_storage, q_str_link);
	}
	return q_str_link;
}

QString Krawler::createUrlStrfromPath(const QString _parent_url_proto, const QString _parent_url_host, const QString _parent_url_dir, QString _qstr_anchor)
{
	QString new_url_str = _qstr_anchor;
	bool no_modify = false;
	
	if (_qstr_anchor.startsWith("/"))
	{
		// check if html ref # is present
		QString tmp_str_anchor = _qstr_anchor;
		if (tmp_str_anchor.contains('#', true) > 0) 
			tmp_str_anchor = tmp_str_anchor.section('#', 0, 0);
		
		// check if local file or not
		QFile local_file(tmp_str_anchor);
		if (local_file.exists())
		{
			new_url_str =  _qstr_anchor;
			no_modify = true;
		}
		else new_url_str = (_parent_url_proto + "://" + _parent_url_host +  _qstr_anchor);
	}
	else {
		// to check if the link has protocol and domain set
		KURL tmp_link_url(_qstr_anchor);  
		
/*		if (tmp_link_url.hasHost() == false) 
			new_url_str = _parent_url_host +  _parent_url_dir + new_url_str;	*/
		
		if (tmp_link_url.protocol() == 0)
			new_url_str = _parent_url_proto + "://" + _parent_url_host +  _parent_url_dir + new_url_str;
	}	
	if (!no_modify)
	{
		KURL full_url(new_url_str);
		full_url.cleanPath();
		new_url_str = full_url.url();
	}
// 	debugMsg("stringWithinQuotes.... _qstr_anchor:");
	return new_url_str;
}

bool Krawler::krawlLink(KURL _embed_url)
{
/*	bool path_check = true;
	bool host_check = true;
	bool host_same = (_embed_url.host() == m_url.host());
	host_check = (!host_same && m_bTraverseOut) || host_same;
	if (host_check)
	{
		QString embed_dir_str = _embed_url.directory(false, false);
		QString m_url_dir_str = m_url.directory(false, false);
		bool parent_same = embed_dir_str.startsWith(m_url_dir_str);
		path_check = (!parent_same && m_bLinkOutsidePath) || parent_same;
	}
	return (host_check && path_check);*/
	bool a = m_bLinkOutsidePath;
	bool b = m_bTraverseOut;
	QString embed_dir_str = _embed_url.directory(false, false);
	QString m_url_dir_str = m_url.directory(false, false);
	bool x = embed_dir_str.startsWith(m_url_dir_str);
	bool y = (_embed_url.host() == m_url.host());
	bool r = (!y && b) || (!x && a && y) || (x && y);
	return r;
}

bool Krawler::isFileTypeExcluded(QString qstr_url)
{
	bool exclude = false;
	if (!m_qstr_list_ExcludedFileTypes.isEmpty())
	{
		QStringList::Iterator it;
		bool _pattern_match = false;
		
		for (it = m_qstr_list_ExcludedFileTypes.begin(); it != m_qstr_list_ExcludedFileTypes.end(); it++)
		{
			QRegExp pattern(".+\\." + (*it) + "\\b");
			if (qstr_url.contains(pattern))
			{
				_pattern_match = true;
				break;
			}
		}
		exclude = (!m_fileType_only && _pattern_match) || (m_fileType_only && !_pattern_match);
	}
	return exclude;
}

QString Krawler::substring(QString src_q_str, int begin_index, int end_index)
{
	int dest_len = end_index - begin_index ;
	QChar* dest_c_str = new QChar[dest_len];
	for (int i=begin_index,j=0; i<end_index; i++,j++)
	{
		dest_c_str[j] = src_q_str[i];
	}
	return QString(dest_c_str,dest_len);
}

QString Krawler::substring(QString src_q_str, int begin_index)
{
	// implements from begin_index to end of string
	int src_q_strlen = src_q_str.length();
	return substring(src_q_str, begin_index, src_q_strlen);
}

void Krawler::debugMsg(QString qstr_debug)
{
// 	kdDebug(9815578)<<"Krawler... "<<qstr_debug<<endl;
}

void Krawler::writeBackContents(QString _parent_file_name, QString _str_text)
{
	// write back the contents
	QFile w_file(_parent_file_name);
	if (w_file.open(IO_WriteOnly))
	{
		QTextStream stream(&w_file);
		stream<<_str_text;
	}
	w_file.close();
}

QString Krawler::readFileContent(QString _file_name)
{
	//read the file
	QFile file(_file_name);
    if (file.open(IO_ReadOnly) == false)
        return 0;
    
	QTextStream stream(&file);
    QString str;
    while (!stream.eof()) 
	{
        str += stream.readLine() + "\n";
	}
	file.close();
	
	return str;
}

void Krawler::replaceUrl(QString _parent_file_name, QString _dl_file_name, QString _replaced_url)
{
	if (_parent_file_name == 0L)
		_parent_file_name = _dl_file_name;
	
// 	debugMsg("_parent_file_name=");
	
	// replace occurrence of _replaced_url in _parent_file_name with _dl_file_name
	QString parent_file_contents = readFileContent(_parent_file_name);
// 	parent_file_contents.replace(_replaced_url, _dl_file_name);
	int begin_index = 0;
	while((begin_index = parent_file_contents.find(_replaced_url, begin_index, true)) != -1)
	{
		if ((parent_file_contents.at(begin_index-1) == '\"' && parent_file_contents.at(begin_index + _replaced_url.length()) == '\"') ||
			(parent_file_contents.at(begin_index-1) == '\'' && parent_file_contents.at(begin_index + _replaced_url.length()) == '\'') )
		{
			parent_file_contents.replace(begin_index, _replaced_url.length(), _dl_file_name);
		}	
		begin_index += _dl_file_name.length();
	}
	
	writeBackContents(_parent_file_name, parent_file_contents);	
}
//   #include "krawler.moc"

QString& Krawler::adjustUrl(QString& file_text, const QString& insert_text, int& init_path_len)
{
	m_stop_krawling = false;
	jumpToNextLink = false;
	jumpToNextUrl = false;
	paused = false;
	
	QStringList links_list = hrefList(file_text, (const KURL )0, false);
	QStringList imgList = imgSrcList(file_text, (const KURL )0, false);
	
	QStringList::Iterator it;
	QString _link, _tmp_link;
	
	for (it = imgList.begin(); it != imgList.end(); it++) 
	{
		_tmp_link = _link = QString(*it);
		if (_link.contains(insert_text) != 0)
			continue;
		_link = _link.insert(init_path_len, insert_text);
		file_text.replace(_tmp_link, _link);
	}	
	
	for (it = links_list.begin(); it != links_list.end(); it++) 
	{
		_tmp_link = _link = QString(*it);
		if (_link.contains(insert_text) != 0)
			continue;
		_link = _link.insert(init_path_len, insert_text);
		file_text.replace(_tmp_link, _link);
	}	
	
	return file_text;
}

bool Krawler::isDynamicContent(QString _file_name)
{
	QStringList::Iterator it;
	bool _pattern_match = false;
	
	for (it = m_dcFileTypes.begin(); it != m_dcFileTypes.end(); it++)
	{
		QRegExp pattern(".+\\." + (*it) + "\\b");
		if (_file_name.contains(pattern))
		{
			_pattern_match = true;
			break;
		}
	}

	return _pattern_match;
}

void Krawler::parseUrl(QStringList& _links, QStringList& _images, QString _url_str)
{
	if (_url_str != 0)
		m_str_url = _url_str;
	if (m_str_url != 0)
	{
		QString str_text = readFileContent(m_str_url);
		_links = hrefList(str_text, (const KURL) 0, false);
		_images = imgSrcList(str_text, (const KURL) 0, false);
	}
}

void Krawler::kurlAttributes(const KURL embedUrl, QString& _parent_url_proto, QString& _parent_url_host, QString& _parent_url_dir)
{
	_parent_url_proto = embedUrl.protocol();
	_parent_url_host = embedUrl.host();
	_parent_url_dir = embedUrl.directory(false, false);
}

void Krawler::post_event(QCustomEvent *e)
{
	QApplication::postEvent(m_object, e);
}

JobOptions::JobOptions()
{
	set_flag = false;
	crawl_mode = -1;
}

JobOptions::~JobOptions()
{}

QString JobOptions::allAttributes()
{
	return QString("crawl mode=%1, crawl_depth=%2, traverseOut=%3, linkOutsidePath=%4, fileTypeList=<" + fileTypeList.join(",") + ">, exclusive_flag=%5, dcTypes=<" + dcTypes.join(",") + ">").arg(crawl_mode).arg(crawl_depth).arg(traverseOut).arg(linkOutsidePath).arg(exclusive_flag);
}

