/***************************************************************************
*   Copyright (C) 2004 by Sayantam Dey                                    *
*   sayantam@gmail.com                                                    *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#ifndef KRAWLER_H
#define KRAWLER_H

#include <kurl.h>
#include <kio/job.h>
#include <ktempfile.h>

#include <qthread.h>
#include <qfile.h>
#include <qvaluevector.h>
#include <qwaitcondition.h>

#include "commonds.h"

typedef QValueList<KTempFile> TempFileList;

/**
@author Sayantam Dey
*/

class Krawler : public QObject, public QThread
{
		Q_OBJECT
	public:
		
		enum KrawlModes
		{
			Default = 0, Offline_Browser, Leech
		};
		
		enum CrawlReturnCodes
		{
			C_OK = 0, C_SKIP, C_FAIL, C_GT_MAX_LEVEL, C_NOT_HTML, C_KILL
		};
		
		enum LinkTypes
		{
			IMG, HREF
		};
		
		Krawler();
		
		Krawler(QString _str_url);
		
		Krawler(QObject* _object, QWidget* _widget, t_LeechJobList _checked_urls, QString _qstr_init_dir);
		
		Krawler(QObject* _object, QWidget* _widget, QString _qstr_init_dir, t_KrawlJobList _job_list);
		
		~Krawler();
		void run();
		// archive helper
		QString& adjustUrl(QString& file_text, const QString& insert_text, int& init_path_len);				 
		//helper + krawler
		void writeBackContents(QString _parent_file_name, QString _str_text);	
		QString readFileContent(QString _file_name);
		void parseUrl(QStringList& _links, QStringList& _images, QString _url_str = 0);
		
	protected:
		
		// methods
		void init(QWidget* _widget, QString _qstr_init_dir);
		
		int  startCrawling(KURL embedUrl, int level, KURL::List, QString _parent_file_name = 0L, uint _index = 0, int _url_type = HREF);
		bool isFileTypeExcluded(QString qstr_url);
		QStringList frameList(const QString& str_text); 
		QStringList hrefList(QString& str_text, const KURL embedUrl, bool decorateUrl = true);
		QStringList imgSrcList(QString& str_text, const KURL embedUrl, bool decorateUrl = true);
		inline QString stringWithinQuotes(const QString _parent_url_proto, const QString _parent_url_host, const QString _parent_url_dir, QString& quoted_text, const int begin_index, bool decorateUrl);
		inline QString substring(QString src_q_str, int begin_index, int end_index);
		inline QString substring(QString src_q_str, int begin_index);
		QString downloadFile(KURL embedUrl, int& result_code, QString& _embed_dir_str, uint _index = 0, int _url_type = HREF);
		bool continueKrawling();
		inline void debugMsg(QString qstr_debug);
		bool krawlLink(KURL _embed_url);
		inline QString createUrlStrfromPath(const QString _parent_url_proto, const QString _parent_url_host, const QString _parent_url_dir, QString path);
		void replaceUrl(QString _parent_file_name, QString _dl_file_name, QString _replaced_url);
		void performCleanup();
		inline void kurlAttributes(const KURL embedUrl, QString& _parent_url_proto, QString& _parent_url_host, QString& _parent_url_dir);
		int singleCrawlJob();
		bool checkAndPause();
		void setJobParameters(JobOptions _job_opt);
		bool ignoreLink(QString _url_str);
		void post_event(QCustomEvent *e);
			
		//attributes
		QWidget *m_widget;
		QObject *m_object;
		int m_iMaxLevel;
		bool m_bTraverseOut;
		bool m_bLinkOutsidePath;
		bool m_stop_krawling;
		bool m_bad_download;
		bool m_download_in_progress;
		bool m_leech_job;
		QString m_qstr_init_dir;
		QMutex m_lock;
		unsigned long m_ul_processed_job_size;
		unsigned long m_ul_last_processed_job_size;
		QStringList m_qstr_list_ExcludedFileTypes;
		QStringList m_dcFileTypes;
		KIO::Job* m_current_kio_job;
		t_KrawlJobList m_job_list;
		QStringList m_ignore_urls;
		
	private:
		KURL m_url;	
		int m_iKrawlMode;
		bool m_fileType_only;
		TempFileList temp_file_list;
		QString m_str_url;
		t_LeechJobList m_checked_Urls;
		QStringList m_combo_urls;
		QString m_DlSkipReason;
		bool jumpToNextLink;
		bool jumpToNextUrl;
		bool paused;
		QMutex m_ig_mutex;
		int dl_url_type;
		int m_index;
			
		enum DownloadReturnCodes 
		{
			DL_OK = 0, DL_SKIP, DL_DIR, DL_FAIL, DL_KILL
		};
		
		bool isDynamicContent(QString _file_name);
		
	protected slots:
		void stopCrawlingClicked();
		void slotResult(KIO::Job *);
		void slotProcessedSize(KIO::Job *, KIO::filesize_t);
		void slotPercent(KIO::Job *, unsigned long);
		void slotSpeed(KIO::Job *, unsigned long);
		void slotInfoMessage(KIO::Job *, const QString &);
	
	public slots:
		void gotoNextLink();
		void gotoNextURL();
		void pauseKrawl(bool);
		void slotIgnoreLink(const QString&);
};

#endif
