/***************************************************************************
 *   Copyright (C) 2004 by Sayantam Dey                                    *
 *   sayantam@gmail.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "krawlerevent.h"

KrawlerEvent::KrawlerEvent()
	: QCustomEvent(EVENT_TYPE)
{
	m_event_type = -1;
}

KIO::Job* KrawlerEvent::getJob()
{
	return m_job;
}

unsigned long KrawlerEvent::getData()
{
	return m_data;
}

int KrawlerEvent::eventType()
{
	return m_event_type;
}
		
KwResultEvent::KwResultEvent(KIO::Job *_job)
	: KrawlerEvent()
{
	m_job = _job;
	m_event_type = RESULT;
}

KwPercentEvent::KwPercentEvent(KIO::Job *_job, unsigned long _data)
	: KrawlerEvent()
{
	m_job = _job;
	m_data = _data;
	m_event_type = PERCENT;
}

KwProcessedEvent::KwProcessedEvent(unsigned long _data)
	: KrawlerEvent()
{
	m_data = _data;
	m_event_type = PROCESSED;
}

KwSpeedEvent::KwSpeedEvent(KIO::Job* _job, unsigned long _data)
	: KrawlerEvent()
{
	m_job = _job;
	m_data = _data;
	m_event_type = SPEED;
}

KwStatusTextEvent::KwStatusTextEvent(const QString& _str)
	: KrawlerEvent()
{
	m_str = _str;
	m_event_type = STATUSTEXT;
}

KwStatusTextEvent::KwStatusTextEvent(KIO::Job* _job, const QString& _str)
	: KrawlerEvent()
{
	m_job = _job;
	m_str = _str;
	m_event_type = INFOMESG;
}

KwParsedLinkEvent::KwParsedLinkEvent(int _index, const QString& _embed_url, 
									 int _level, const QStringList& _links, 
									 const QStringList& _imgs)
	: KrawlerEvent()
{
	m_index = _index;
	m_embed_url = _embed_url;
	m_level = _level;
	m_links = _links;
	m_imgs = _imgs;
	m_event_type = PARSEDLINK;
}		

KwImgStatsEvent::KwImgStatsEvent(int _index, int _img_list_sz)
	: KrawlerEvent()
{
	m_index = _index;
	m_img_list_sz = _img_list_sz;
	m_event_type = IMGSTATS;
}

KwLinkStatsEvent::KwLinkStatsEvent(int _index, int _link_list_sz)
	: KrawlerEvent()
{
	m_index = _index;
	m_link_list_sz = _link_list_sz;
	m_event_type = LINKSTATS;
} 

KwFinishedEvent::KwFinishedEvent(int _data)
	: KrawlerEvent()
{
	m_data = _data;
	m_event_type = FINISHED;
}

KwErrorMesgEvent::KwErrorMesgEvent(const QString& _str)
	: KrawlerEvent()
{
	m_str = _str;
	m_event_type = ERRORMESG;
}

KwCurrentDlUrlEvent::KwCurrentDlUrlEvent(const QString& _url_str, int _url_type)
	: KrawlerEvent()
{
	m_url_str = _url_str;
	m_url_type = _url_type;
	m_event_type = CURRENTURL;
}

KwFileDlComplete::KwFileDlComplete(int _data, const QString& _url_str, const QString& _file_name)
	: KrawlerEvent()
{
	m_data = _data;
	m_url_str = _url_str;
	m_file_name = _file_name;
	m_event_type = DLCOMPLETE; 
}

KwNextJobEvent::KwNextJobEvent(int _data, const KURL& _url)
	: KrawlerEvent()
{
	m_data = _data;
	m_url = _url;
	m_event_type = NEXTJOB;
}

KwLeechedLinkEvent::KwLeechedLinkEvent()
	: KrawlerEvent()
{
	m_event_type = LEECHED;
}

KwSelectedDlCompleteEvent::KwSelectedDlCompleteEvent()
	: KrawlerEvent()
{
	m_event_type = SELDLCOMPLETE;
}

KwMarkerEvent::KwMarkerEvent(uint _data, uint _index, int _url_type)
	: KrawlerEvent()
{
	m_data = _data;
	m_index = _index;
	m_url_type = _url_type;
	m_event_type = MARKER;
}
