/***************************************************************************
 *   Copyright (C) 2004 by Sayantam Dey                                    *
 *   sayantam@gmail.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef KRAWLEREVENT_H
#define KRAWLEREVENT_H

#include <kurl.h>
#include <kio/job.h>
#include <qevent.h>

/**
@author Sayantam Dey
*/

class KrawlerEvent : public QCustomEvent
{
	protected:
		int m_event_type;
		KIO::Job *m_job;
		unsigned long m_data;
		 
	public:
		enum { EVENT_TYPE = (QEvent::User + 1456) };
		enum { RESULT, PERCENT, PROCESSED, SPEED, STATUSTEXT, PARSEDLINK,
			IMGSTATS, LINKSTATS, FINISHED, ERRORMESG, INFOMESG, CURRENTURL, DLCOMPLETE,
			NEXTJOB, LEECHED, MARKER, SELDLCOMPLETE};
		
		KrawlerEvent();
		KIO::Job*   getJob();
		unsigned long getData();
		int eventType();
};

class KwResultEvent : public KrawlerEvent
{
	public:
		KwResultEvent(KIO::Job *_job);
};
		
class KwPercentEvent : public KrawlerEvent
{
	public:
		KwPercentEvent(KIO::Job *_job, unsigned long _data);
};
	
class KwProcessedEvent : public KrawlerEvent
{
	public:
		KwProcessedEvent(unsigned long _data);
};
	
class KwSpeedEvent : public KrawlerEvent
{
	public:
		KwSpeedEvent(KIO::Job *, unsigned long);
};
		
class KwStatusTextEvent : public KrawlerEvent
{
	public:
		QString m_str;
	
		KwStatusTextEvent(const QString&);
		KwStatusTextEvent(KIO::Job *, const QString&);
};

class KwParsedLinkEvent : public KrawlerEvent
{
	public:
		int m_index;
		QString m_embed_url;
		int m_level;
		QStringList m_links;
		QStringList m_imgs;
		
		KwParsedLinkEvent(int, const QString&, int, const QStringList&, const QStringList&);		
};		

class KwImgStatsEvent : public KrawlerEvent
{
	public:
		int m_index;
		int m_img_list_sz;
		
		KwImgStatsEvent(int, int);
};

class KwLinkStatsEvent : public KrawlerEvent
{
	public:
		int m_index;
		int m_link_list_sz;
		
		KwLinkStatsEvent(int, int);
};

class KwFinishedEvent : public KrawlerEvent
{
	public:
		KwFinishedEvent(int);
};

class KwErrorMesgEvent : public KrawlerEvent
{
	public:
		QString m_str;
		
		KwErrorMesgEvent(const QString&);
};

class KwCurrentDlUrlEvent : public KrawlerEvent
{
	public:
		QString m_url_str;
		int m_url_type;
		
		KwCurrentDlUrlEvent(const QString&, int);
};

class KwFileDlComplete : public KrawlerEvent
{
	public:
		QString m_url_str;
		QString m_file_name; 
		
		KwFileDlComplete(int, const QString&, const QString&);
};
		
class KwNextJobEvent : public KrawlerEvent
{
	public:
		KURL m_url;
		
		KwNextJobEvent(int, const KURL&);
};

class KwLeechedLinkEvent : public KrawlerEvent
{
	public:
		KwLeechedLinkEvent();
};

class KwSelectedDlCompleteEvent : public KrawlerEvent
{
	public:
		KwSelectedDlCompleteEvent();
};			

class KwMarkerEvent : public KrawlerEvent
{
	public:
		unsigned int m_index;
		int m_url_type;
		
		KwMarkerEvent(uint, uint, int);
};


#endif
