/***************************************************************************
 *   Copyright (C) 2004 by Sayantam Dey                                    *
 *   sayantam@gmail.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <klocale.h>
#include <kdebug.h>
#include <kiconloader.h>

#include <qlayout.h>

#include "krawlmainwidget.h"

KrawlMainWidget::KrawlMainWidget(QWidget *parent, const char *name, WId _container_id)
 : QWidget(parent, name), first_time(true), m_op_mode(-1)
{
	v_layout_0 = new QVBoxLayout(0, 1, 1, "v-layout-0");
	//page options
	qso_frame = new QuickSetOptions();
 	connect(qso_frame, SIGNAL(changedKrawlMode(const QString& )), this, SLOT(changeKrawlDisp(const QString& )));
	
	// quick set button	
	btn_qso_popup = new KPushButton(this, "quick set btn");
    btn_qso_popup->setFocusPolicy( KPushButton::NoFocus );
    btn_qso_popup->setToggleButton( TRUE );
	btn_qso_popup->setIconSet(KGlobal::iconLoader()->loadIconSet("html", KIcon::NoGroup, 16));
	btn_qso_popup->setText("&Settings");
	connect(btn_qso_popup, SIGNAL(clicked()), this, SLOT(qsoBtnClicked()));
	
	m_multi_job_handler = new MultiJobHandler(this, "multiple job handler", _container_id);
	h_layout_0 = new QHBoxLayout(0, 1, 1, "h-layout-0");
	h_layout_0->addWidget(btn_qso_popup);
	h_layout_0->addWidget(m_multi_job_handler);
	v_layout_0->addLayout(h_layout_0);

	splitter = new QSplitter(this, "splitter");
    splitter->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)7, 0, 0, splitter->sizePolicy().hasHeightForWidth() ) );
    splitter->setFrameShape( QSplitter::StyledPanel );
    splitter->setFrameShadow( QSplitter::Sunken );
    splitter->setOrientation( QSplitter::Horizontal );
	v_layout_0->addWidget(splitter);
	int splitter_width = parent->width();
// 	debugMsg(QString("splitter width:%1").arg(splitter_width));
	QWidget* privateLayoutWidget = new QWidget( splitter, "layout12" );
	QVBoxLayout* layout12 = new QVBoxLayout( privateLayoutWidget, 1, 1, "layout12"); 
	QWidget* privateLayoutWidget_2 = new QWidget( splitter, "layout14" );	
	QVBoxLayout* layout14 = new QVBoxLayout( privateLayoutWidget_2, 1, 1, "layout14"); 
	
// 	file manager
	m_file_manager = new FileManager(privateLayoutWidget, "file manager");
	layout12->addWidget(m_file_manager);
	m_url_display = new UrlDisplay((5*splitter_width/7), privateLayoutWidget_2, "url display" );
	m_url_display->setMultiJobHandlerInst(m_multi_job_handler);
	m_url_display->setPageOptions(qso_frame);
	m_multi_job_handler->setPageOptions(qso_frame);
// 	layout14->addWidget(m_multi_job_handler); 
	layout14->addWidget(m_url_display);
	
	QValueList<int> size_list;
	size_list.append((int) (splitter_width/7));
	size_list.append((int) (5*splitter_width/7));
	splitter->setSizes(size_list);
	
	m_info_panel = new InfoPanel(this);
	v_layout_0->addWidget(m_info_panel);
	
	grid_layout = new QGridLayout(this, 1, 1, 1, 1, "grid_layout");
	grid_layout->addLayout(v_layout_0, 0, 0);
	
	this->clearWState( WState_Polished );
	
	m_url_display->setInfoPanelInst(m_info_panel);
	connect(m_url_display, SIGNAL(ignoreLink(const QString& )), SIGNAL(ignoreLink(const QString&)));
	
	m_multi_job_handler->setInfoPanelInst(m_info_panel);
	connect(m_multi_job_handler, SIGNAL(jumpToNextLink()), SIGNAL(jumpToNextLink()));
	connect(m_multi_job_handler, SIGNAL(jumpToNextUrl()), SIGNAL(jumpToNextUrl()));
	connect(m_multi_job_handler, SIGNAL(pauseKrawl(bool )), SIGNAL(pauseKrawl(bool)));
	connect(m_multi_job_handler, SIGNAL(startLeech()), SIGNAL(startLeech()));
	connect(m_multi_job_handler, SIGNAL(startMultiJob()), SIGNAL(startMultiJob()));
	
	init_widget();
	first_time = true;
	loadSavedDefaults();
	

	setMinimumSize(sizeHint());
}


KrawlMainWidget::~KrawlMainWidget()
{
	debugMsg("KrawlMainWidget destructor called");
}

QString KrawlMainWidget::baseDlDirectory()
{
	return m_file_manager->baseDlDirectory();
}

int KrawlMainWidget::crawlDepth()
{
	return qso_frame->crawlDepth();
}

bool KrawlMainWidget::traverseOut()
{
	return qso_frame->traverseOut();
}

bool KrawlMainWidget::linkOutsidePath()
{
	return qso_frame->linkOutsidePath();
}

void KrawlMainWidget::clearDlLabel()
{	
	m_info_panel->clearDlLabel();
}

void KrawlMainWidget::init_widget()
{
	m_info_panel->init_widget();
	m_multi_job_handler->init_widget(krawlMode());
	m_url_display->init_widget();
	m_file_manager->initFileView();
	first_time = false;
}

void KrawlMainWidget::setTotalDlSize(unsigned long value)
{
	m_info_panel->setTotalDlSize(value);
}

void KrawlMainWidget::setDlSpeed(unsigned long value)
{
	m_info_panel->setDlSpeed(value);
}

void KrawlMainWidget::loadSavedDefaults(bool _apply)
{
	if ( first_time || (!first_time && _apply) )
	{
		m_file_manager->loadSavedDefaults();	
		qso_frame->loadSettings();
	}
}

void KrawlMainWidget::saveSettings()
{
	if (rememberLastBaseDir)
	{
		m_file_manager->saveSettings();
	}
}

void KrawlMainWidget::setStatusMsg(QString qstr_status_msg)
{
	m_info_panel->setStatusMsg(qstr_status_msg);
}

void KrawlMainWidget::setLinkStats(int index, int _total_steps)
{
	m_info_panel->setLinkStats(index, _total_steps);
}

void KrawlMainWidget::setImgStats(int index, int _total_steps)
{
	m_info_panel->setImgStats(index, _total_steps);
}


void KrawlMainWidget::setCurrentDlUrl(QString _qstr_url, int url_type)
{
	m_info_panel->setCurrentDlUrl(_qstr_url);
	m_url_display->markCurrentDownload(_qstr_url, url_type);
}

void KrawlMainWidget::changeFileView(const QString& _url_str)
{
	m_file_manager->changeFileView(_url_str);
}

int KrawlMainWidget::krawlMode()
{
	return qso_frame->krawlMode();
}

bool KrawlMainWidget::fileTypeList(QStringList &_type_list)
{
	return qso_frame->fileTypeList(_type_list);
}

QStringList KrawlMainWidget::dcTypeList()
{
	return qso_frame->dcTypeList();
}

t_LeechJobList KrawlMainWidget::getCheckedUrls()
{
	return m_url_display->getCheckedUrls();
}

QStringList KrawlMainWidget::getComboUrls()
{
	return m_multi_job_handler->getComboUrls();
}

void KrawlMainWidget::krawlRunning(bool _flag)
{
	m_multi_job_handler->krawlRunning(_flag);
}

void KrawlMainWidget::setOpMode(int mode)
{
	m_multi_job_handler->setOpMode(mode);
}

void KrawlMainWidget::changeKrawlDisp(const QString& _mode_str)
{
	m_info_panel->changeKrawlDisp(_mode_str);
}

void KrawlMainWidget::qsoBtnClicked()
{
	if (btn_qso_popup->isOn())
	{
		QPoint p(QWidget::mapToGlobal(QPoint(btn_qso_popup->x(), btn_qso_popup->y() + btn_qso_popup->height())));
		showSettingsPopup(p);
	}
}

void KrawlMainWidget::showSettingsPopup(QPoint _p, QuickSetOptions* qso_frame_ptr, bool _map_to_parent)
{
	if (qso_frame_ptr == 0)
		qso_frame_ptr = qso_frame;
	OptionsPopupFrame* popup_fr = new OptionsPopupFrame(this, "popup_fr");
	connect(popup_fr, SIGNAL(popupClosed()), btn_qso_popup, SLOT(toggle()));
	if (_map_to_parent)
		_p = popup_fr->mapFromGlobal(_p);
	qso_frame_ptr->reparent(popup_fr, 0, _p);
	popup_fr->setMainWidget(qso_frame_ptr);
	popup_fr->exec(_p);
	debugMsg("control reaches here");
	if (btn_qso_popup->isOn())	
		btn_qso_popup->toggle();
}

void KrawlMainWidget::addLinkInfo(int _index, QString q_str_url, int link_depth, const QStringList& _links, const QStringList& _images)
{
	m_url_display->addLinkInfo(_index, q_str_url, link_depth, _links, _images);
}

void KrawlMainWidget::progressUpdate(int percent)
{
	m_info_panel->progressUpdate(percent);
}

QString KrawlMainWidget::updateComboIndex(int index)
{
	return m_multi_job_handler->updateComboIndex(index);
}

KURL KrawlMainWidget::branchUrl()
{
	return m_file_manager->branchUrl();
}

void KrawlMainWidget::debugMsg(QString _str)
{
	kdDebug(9815578)<<"KrawlMainWidget.... "<<_str<<endl;
}

void KrawlMainWidget::doneLeech()
{
// 	m_multi_job_handler->enableSave(true);
}

t_KrawlJobList KrawlMainWidget::jobMap()
{
	return m_multi_job_handler->m_KrawlJobList;
}

/*$SPECIALIZATION$*/
OptionsPopupFrame::OptionsPopupFrame(QWidget* parent, const char* name)
 : KPopupFrame (parent, name)
{
}

OptionsPopupFrame::~ OptionsPopupFrame()
{
	kdDebug(9815578)<<"OptionsPopupFrame destructor called"<<endl;
}

void OptionsPopupFrame::closeEvent(QCloseEvent* e)
{
	emit popupClosed();
	close(0);
	e->accept();
}


#include "krawlmainwidget.moc"
