/***************************************************************************
 *   Copyright (C) 2004 by Sayantam Dey                                    *
 *   sayantam@gmail.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef KRAWLMAINWIDGET_H
#define KRAWLMAINWIDGET_H

#include <kurl.h>
#include <kdatetbl.h>
#include <kpushbutton.h>

#include "krawlconfig.h"
#include "dropwindow.h"
#include "quicksetoptions.h"
#include "krawler.h"
#include "filemanager.h"
#include "infopanel.h"
#include "multijobhandler.h"
#include "urldisplay.h"
#include "commonds.h"

/**
@author Sayantam Dey
*/
class OptionsPopupFrame : public KPopupFrame
{
Q_OBJECT
	
public:
	OptionsPopupFrame(QWidget *parent = 0, const char *name = 0);
	~OptionsPopupFrame();
	virtual void closeEvent(QCloseEvent* e);

signals:
	void popupClosed();
};

class KrawlMainWidget : public QWidget
{
Q_OBJECT
public:
    KrawlMainWidget(QWidget *parent = 0, const char *name = 0, WId _container_id = -1);

    ~KrawlMainWidget();

	/*$PUBLIC_FUNCTIONS$*/
	
	QString baseDlDirectory();
	KURL branchUrl();
	int crawlDepth();
	bool traverseOut();
	bool linkOutsidePath();
	void addLinkInfo(int _index, QString q_str_url, int link_depth, const QStringList& _links, const QStringList& _images);
	void init_widget();
	void setTotalDlSize(unsigned long value);
	void setDlSpeed(unsigned long value);
	void setStatusMsg(QString qstr_status_msg);
	void setLinkStats(int index, int number);
	void setImgStats(int index, int number);
	void setCurrentDlUrl(QString _qstr_url, int url_type);
	int krawlMode();
  	bool fileTypeList(QStringList &_type_list);
	QStringList dcTypeList();
	void clearDlLabel();
	t_LeechJobList getCheckedUrls();
	QStringList getComboUrls();
	QString updateComboIndex(int index);
	void krawlRunning(bool _flag);
	void setOpMode(int mode);
	void loadSavedDefaults(bool _apply = false);
  	void saveSettings();	
	void progressUpdate(int percent);
	void doneLeech();
	t_KrawlJobList jobMap();
	
	FileManager* m_file_manager;	
	MultiJobHandler* m_multi_job_handler;
	UrlDisplay* m_url_display;
	InfoPanel* m_info_panel;
	
protected:
	QuickSetOptions* qso_frame;
	bool saveComboUrls;
	bool rememberLastBaseDir;
	bool first_time;
	int m_op_mode;
	bool krawl_paused;
	QuickSetOptions* m_qso_holder;
    KPushButton* btn_qso_popup;
	
	//gui-items
	QVBoxLayout* v_layout_0;
	QSpacerItem* spacer;
	QHBoxLayout* h_layout_0;
	QGridLayout* grid_layout;
	QSplitter* splitter;
				
	/*$PROTECTED_FUNCTIONS$*/
	void debugMsg(QString _str);
	void showSettingsPopup(QPoint _p, QuickSetOptions* _ptr = 0, bool _map_to_parent = false);
	void changeFileView(const QString& _url_str);

protected slots:
	void qsoBtnClicked();
	void changeKrawlDisp(const QString& _mode_str);	
	
signals:
	void startMultiJob();
	void startLeech();
	void jumpToNextLink();
	void jumpToNextUrl();
	void pauseKrawl(bool);
	void ignoreLink(const QString& );
};

#endif
