/***************************************************************************
 *   Copyright (C) 2004 by Sayantam Dey                                    *
 *   sayantam@gmail.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "krawlsite.h"

#include <kkeydialog.h>
#include <kurlrequesterdlg.h>
#include <kconfig.h>
#include <kurl.h>
#include <kdebug.h>
#include <kedittoolbar.h>
#include <kaction.h>
#include <kstdaction.h>
#include <kdebug.h>
#include <klibloader.h>
#include <kmessagebox.h>
#include <kstatusbar.h>
#include <klocale.h>
#include <kiconloader.h>

#include <qtooltip.h>
#include <qevent.h>

#include "commonds.h"

KrawlSite::KrawlSite()
    : KParts::MainWindow( 0L, "KrawlSite", 0)
{
    // set the shell's ui resource file
    setXMLFile("krawlsite_shell.rc");

    // then, setup our actions
    setupActions();

    // and a status bar
    statusBar()->show();
	
	// hide the damn toobar
	this->toolBar("")->hide();

    // this routine will find and load our Part.  it finds the Part by
    // name which is a bad idea usually.. but it's alright in this
    // case since our Part is made for this Shell
    KLibFactory *factory = KLibLoader::self()->factory("libkrawlsitepart");
    if (factory)
    {
        // now that the Part is loaded, we cast it to a Part to get
        // our hands on it
        m_part = static_cast<KParts::ReadOnlyPart *>(factory->create(this,
                                "krawlsite_part", "KParts::ReadOnlyPart" ));

        if (m_part)
        {
            // tell the KParts::MainWindow that this is indeed the main widget
            setCentralWidget(m_part->widget());

            // and integrate the part's GUI with the shell's
            createGUI(m_part);
 		}
    }
    else
    {
        // if we couldn't find our Part, we exit since the Shell by
        // itself can't do anything useful
        KMessageBox::error(this, i18n("Could not find our part!"));
		kdDebug(9815578)<<"error:"<<KLibLoader::self()->lastErrorMessage()<<endl;
		
        kapp->quit();
        // we return here, cause kapp->quit() only means "exit the
        // next time we enter the event loop...
        return;
    }

    // apply the saved mainwindow settings, if any, and ask the mainwindow
    // to automatically save settings if changed: window size, toolbar
    // position, icon size, etc.
    setAutoSaveSettings();
	
	connect(m_part, SIGNAL(started(KIO::Job*)), this, SLOT(slotKrawlBegin()));
	connect(m_part, SIGNAL(completed()), this, SLOT(slotKrawlEnd()));
	
	//trying for dock functionality
	sys_tray = new KSystemTray(this,"KrawlSite");
	connect(sys_tray, SIGNAL(quitSelected()), this, SLOT(quit()));
	sys_tray->setPixmap(sys_tray->loadIcon("krawlsite"));
	QToolTip::add(sys_tray, tr("KrawlSite-web crawler"));
	makeContextMenu(sys_tray->contextMenu());
	sys_tray->show();
}

KrawlSite::~KrawlSite()
{
	kdDebug(9815578)<<"KrawlSite destructor called."<<endl;
	
	delete m_part;
	m_part = 0;
}

void KrawlSite::makeContextMenu(KPopupMenu *_ptr)
{
	int index = 1;
/*	_ptr->insertItem(QIconSet(KGlobal::iconLoader()->loadIcon("folder_html", KIcon::NoGroup, 16)), i18n("Drop Target"), KrawlSiteTray::KST_DROP, index++);*/
	_ptr->insertItem(QIconSet(KGlobal::iconLoader()->loadIcon("edit_add", KIcon::NoGroup, 16)), i18n("Add URL"), KrawlSiteTray::KST_ADD, index++);
	_ptr->insertItem(QIconSet(KGlobal::iconLoader()->loadIcon("editclear", KIcon::NoGroup, 16)), i18n("Clear URLs"), KrawlSiteTray::KST_CLEAR, index++);
	_ptr->insertItem(QIconSet(KGlobal::iconLoader()->loadIcon("player_play", KIcon::NoGroup, 16)), i18n("Start"), KrawlSiteTray::KST_START, index++); 
	_ptr->insertItem(QIconSet(KGlobal::iconLoader()->loadIcon("player_pause", KIcon::NoGroup, 16)), i18n("Pause/Resume"), KrawlSiteTray::KST_PAUSE, index++); _ptr->setItemEnabled(KrawlSiteTray::KST_PAUSE, false);
	_ptr->insertItem(QIconSet(KGlobal::iconLoader()->loadIcon("player_fwd", KIcon::NoGroup, 16)), i18n("Next Link"), KrawlSiteTray::KST_NEXT_LINK, index++); _ptr->setItemEnabled(KrawlSiteTray::KST_NEXT_LINK, false);
	_ptr->insertItem(QIconSet(KGlobal::iconLoader()->loadIcon("player_end", KIcon::NoGroup, 16)), i18n("Next URL"), KrawlSiteTray::KST_NEXT_URL, index++); _ptr->setItemEnabled(KrawlSiteTray::KST_NEXT_URL, false);
	_ptr->insertItem(QIconSet(KGlobal::iconLoader()->loadIcon("player_stop", KIcon::NoGroup, 16)), i18n("Stop"), KrawlSiteTray::KST_STOP, index++); _ptr->setItemEnabled(KrawlSiteTray::KST_STOP, false);
	connect(_ptr, SIGNAL(activated(int)), this, SLOT(slotContextMenu(int )));	
}

void KrawlSite::slotKrawlBegin()
{
	toggleContextMenu(true);
}

void KrawlSite::slotKrawlEnd()
{
	toggleContextMenu(false);
}

void KrawlSite::toggleContextMenu(bool _flag)
{
	sys_tray->contextMenu()->setItemEnabled(KrawlSiteTray::KST_PAUSE, _flag);	
	sys_tray->contextMenu()->setItemEnabled(KrawlSiteTray::KST_NEXT_LINK, _flag);	
	sys_tray->contextMenu()->setItemEnabled(KrawlSiteTray::KST_NEXT_URL, _flag);	
	sys_tray->contextMenu()->setItemEnabled(KrawlSiteTray::KST_STOP, _flag);	
}

void KrawlSite::slotContextMenu(int id)
{
	if (id == KrawlSiteTray::KST_STOP)
	{
		toggleContextMenu(false);
	}
	KApplication::postEvent(m_part, new QCustomEvent(static_cast<QEvent::Type>(QCustomEvent::User+9999), &id));
}

void KrawlSite::load(const KURL& url)
{
    m_part->openURL( url );
}

void KrawlSite::setupActions()
{
//    	KStdAction::openNew(this, SLOT(fileNew()), actionCollection());
//     KStdAction::open(this, SLOT(fileOpen()), actionCollection());

    KStdAction::quit(this, SLOT(quit()), actionCollection());

    m_toolbarAction = KStdAction::showToolbar(this, SLOT(optionsShowToolbar()), actionCollection());
    m_statusbarAction = KStdAction::showStatusbar(this, SLOT(optionsShowStatusbar()), actionCollection());

    KStdAction::keyBindings(this, SLOT(optionsConfigureKeys()), actionCollection());
    KStdAction::configureToolbars(this, SLOT(optionsConfigureToolbars()), actionCollection());
}

void KrawlSite::saveProperties(KConfig* /*config*/)
{
    // the 'config' object points to the session managed
    // config file.  anything you write here will be available
    // later when this app is restored
	
}

void KrawlSite::readProperties(KConfig* /*config*/)
{
    // the 'config' object points to the session managed
    // config file.  this function is automatically called whenever
    // the app is being restored.  read in here whatever you wrote
    // in 'saveProperties'
}


void KrawlSite::fileNew()
{
    // this slot is called whenever the File->New menu is selected,
    // the New shortcut is pressed (usually CTRL+N) or the New toolbar
    // button is clicked

    // About this function, the style guide (
    // http://developer.kde.org/documentation/standards/kde/style/basics/index.html )
    // says that it should open a new window if the document is _not_
    // in its initial state.  This is what we do here..
    if ( ! m_part->url().isEmpty() )
    {
        (new KrawlSite)->show();
    };
}


void KrawlSite::optionsShowToolbar()
{
    // this is all very cut and paste code for showing/hiding the
    // toolbar
    if (m_toolbarAction->isChecked())
        toolBar()->show();
    else
        toolBar()->hide();
}

void KrawlSite::optionsShowStatusbar()
{
    // this is all very cut and paste code for showing/hiding the
    // statusbar
    if (m_statusbarAction->isChecked())
        statusBar()->show();
    else
        statusBar()->hide();
}

void KrawlSite::optionsConfigureKeys()
{
    KKeyDialog::configure(actionCollection(), this);
}

void KrawlSite::optionsConfigureToolbars()
{
#if defined(KDE_MAKE_VERSION)
# if KDE_VERSION >= KDE_MAKE_VERSION(3,1,0)
    saveMainWindowSettings(KGlobal::config(), autoSaveGroup());
# else
    saveMainWindowSettings(KGlobal::config() );
# endif
#else
    saveMainWindowSettings(KGlobal::config() );
#endif

    // use the standard toolbar editor
    KEditToolbar dlg(factory());
    connect(&dlg, SIGNAL(newToolbarConfig()),
            this, SLOT(applyNewToolbarConfig()));
    dlg.exec();
}

void KrawlSite::applyNewToolbarConfig()
{
#if defined(KDE_MAKE_VERSION)
# if KDE_VERSION >= KDE_MAKE_VERSION(3,1,0)
    applyMainWindowSettings(KGlobal::config(), autoSaveGroup());
# else
    applyMainWindowSettings(KGlobal::config());
# endif
#else
    applyMainWindowSettings(KGlobal::config());
#endif
}

void KrawlSite::fileOpen()
{
    // this slot is called whenever the File->Open menu is selected,
    // the Open shortcut is pressed (usually CTRL+O) or the Open toolbar
    // button is clicked
    KURL url =
        KURLRequesterDlg::getURL( QString::null, this );

    if (url.isEmpty() == false)
    {
        // About this function, the style guide (
        // http://developer.kde.org/documentation/standards/kde/style/basics/index.html )
        // says that it should open a new window if the document is _not_
        // in its initial state.  This is what we do here..
        if ( m_part->url().isEmpty() )
        {
            // we open the file in this window...
            load( url );
        }
        else
        {
			// we open the file in a new window...
            KrawlSite* newWin = new KrawlSite;
            newWin->load( url );
            newWin->show();
        }
    }
}

void KrawlSite::quit()
{
	deleteLater();
	kapp->quit();
}

void KrawlSite::closeEvent(QCloseEvent* e)
{
// 	setShown(false);
	if (!e->spontaneous() 	&& !kapp->sessionSaving() && this->isShown())
	{
		e->ignore();
		KMessageBox::information(this, i18n("<qt>Closing the window will keep Krawlsite running in the system tray. To exit the application, use <B>Quit</B> from the menu or Krawlsite tray-icon.</qt>"), i18n("Docking in System Tray"), i18n("Do not show this dialog to me again"));
		hide();
	}
	else
	{
		this->quit();
	}
}


#include "krawlsite.moc"
