/***************************************************************************
 *   Copyright (C) 2004 by Sayantam Dey                                    *
 *   sayantam@gmail.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <kdebug.h>
#include <kinstance.h>
#include <kstdaction.h>
#include <kactionclasses.h>
#include <kurlrequesterdlg.h>
#include <klocale.h>
#include <kshortcut.h>
#include <kfiletreeview.h>
#include <kdialogbase.h>
#include <krun.h>
#include <kfiledialog.h>
#include <kfileitem.h>
#include <ktar.h>
#include <kmessagebox.h>
#include <kio/global.h>
#include <kiconloader.h>
#include <klineedit.h>
#include <kapplication.h>

#include <qlistview.h>
#include <qpoint.h>
#include <qdir.h>

#include "krawlsite_part.h"
#include "multijobhandler.h"
#include "krawlerevent.h"

KrawlSitePart::KrawlSitePart( QWidget *parentWidget, const char *,
                                  QObject *parent, const char *name )
    : KParts::ReadOnlyPart(parent, name)
{
    // we need an instance
    setInstance( KrawlSitePartFactory::instance() );
	
	// this should be your custom internal widget
    main_widget = new KrawlMainWidget(parentWidget,"main", parentWidget->winId()); 
	connect(main_widget, SIGNAL(startMultiJob()), this, SLOT(multiJobHandler()));
	connect(main_widget, SIGNAL(startLeech()), this, SLOT(leechHandler()));
	connect(main_widget->m_multi_job_handler, SIGNAL(stopCrawling()), this, SLOT(stopCrawling()));
	connect(main_widget->m_multi_job_handler, SIGNAL(createArchive()), this, SLOT(slotCreateArchive()));
	connect(main_widget->m_multi_job_handler, SIGNAL(showConfigure()), this, SLOT(showSettingsDlg()));
	connect(main_widget->m_multi_job_handler, SIGNAL(showLog()), this, SLOT(slotShowLogWindow()));
	
    // notify the part that this is our internal widget
    setWidget(main_widget);
	
	// set our XML-UI resource file
    setXMLFile("krawlsite_part.rc");

    // create our actions
	
	m_stopAction = new KAction(i18n("S&top Krawling"), "stop", KShortcut(Qt::Key_Escape), this, SLOT(stopCrawling()), actionCollection(), "stop");
	m_stopAction->setEnabled(false);
	main_widget->m_multi_job_handler->setStopBtnState( false );
	m_startAction = new KAction(i18n("&Start Krawling"), "start", 0, this, SLOT(goClicked()), actionCollection(), "start");
	
	m_settingsAction = new KAction(i18n("Configure KrawlSite..."), "configure", 0, this, SLOT(showSettingsDlg()), actionCollection(), "settings");
	
	m_archiveAction = new KAction(i18n("Arch&ive Files"), "tar", 0, this, SLOT(slotCreateArchive()), actionCollection(), "archive");
	
	m_showLogWin = new KAction(i18n("&Log Window"), "log", 0, this, SLOT(slotShowLogWindow()), actionCollection(), "show_log");
	
// 	m_clearLocationBar = new KAction(i18n("C&lear Bar"), "locationbar_erase", 0, this, SLOT(clearUrlBar()), actionCollection(), "clear_location");
	
/*	m_UrlListCombo = new KURLComboBox(KURLComboBox::Both, true, parentWidget,"url_combo");
	m_UrlListCombo->setAutoCompletion(KGlobalSettings::CompletionAuto);
	m_UrlListCombo->setContextMenuEnabled(true);
	connect(m_UrlListCombo, SIGNAL(returnPressed()), this, SLOT(comboReturnPressed()));
	combo_action = new KWidgetAction(m_UrlListCombo, i18n("&Location Bar"),0,0,0,actionCollection(),"toolbar_url_combo"); 
	combo_action->setShortcutConfigurable( false );
	combo_action->setAutoSized( true );*/
    
	m_goAction = new KAction(i18n("Start krawling"), "go", 0, this, SLOT(goClicked()), actionCollection(), "go_url");
	
	//no progress windows please
	setProgressInfoEnabled(false);
	
	setStatusText(i18n("Ready"));
	
	config_dlg = 0;
	m_current_kio_job = 0;
	m_krawler = 0;
	m_log_win = 0;
	m_logBrowser = 0;
	m_ArchiveWidget = 0;
	m_archive_win = 0;
	m_notify_win = 0;
	loadSettings();
	main_widget->loadSavedDefaults();
}


KrawlSitePart::~KrawlSitePart()
{
	debugMsg("krawlsite_part destructor called");
	delete m_krawler;
	m_krawler = 0;
	delete config_dlg;
	config_dlg = 0;
	delete m_archive_win;
	m_archive_win = 0;
	delete m_log_win;
	m_log_win = 0;
	delete m_notify_win;
	m_notify_win = 0;
/*	delete main_widget;
	main_widget = 0;*/
	main_widget->saveSettings();
	if (KrawlConfigSkel::self() != 0)
		KrawlConfigSkel::writeConfig();
// 	delete KrawlConfigSkel::self();
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////  OVERRIDDEN METHOD(S) 
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool KrawlSitePart::openFile()
{
	insertUrl(m_url);
	emit setWindowCaption(m_url.prettyURL());
	return true;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////  SLOTS 
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool KrawlSitePart::openURL(const KURL& _url)
{
	m_url = _url;
	openFile();
	return true;
}

void KrawlSitePart::goClicked()
{
	multiJobHandler();
}

void KrawlSitePart::slotCurrentDlUrl(QString _str_url, int _url_type)
{
	main_widget->setCurrentDlUrl(_str_url, _url_type);
}

void KrawlSitePart::slotParsedLinkStats(int _index, QString _url, int _level, const QStringList& _Links, const QStringList& _Images)
{
	main_widget->addLinkInfo(_index, _url, _level, _Links, _Images);
}

void KrawlSitePart::slotSetImgStats(int _index, int _num)
{
	main_widget->setImgStats(_index, _num);	
}

void KrawlSitePart::slotSetLinkStats(int _index, int _num)
{
	main_widget->setLinkStats(_index, _num);
}

void KrawlSitePart::slotFinished(int result_code)
{
	debugMsg("done krawling");    
// 	main_widget->m_multi_job_handler->m_KrawlJobList.clear();
	main_widget->saveSettings();
	main_widget->loadSavedDefaults();
	saveSettings();
	loadSettings();
	////////
// 	KApplication::kApplication()->lock();
	m_stopAction->setEnabled(false); // ------------------
	main_widget->m_multi_job_handler->setStopBtnState( false );
// 	KApplication::kApplication()->unlock();
	////////
	if (result_code == Krawler::C_KILL)
	{
		setStatusText(i18n("Terminated by user"));
	}
	else 
	{
		setStatusText(i18n("Done"));
		main_widget->progressUpdate(100);
	}
	main_widget->setDlSpeed(0);
	main_widget->clearDlLabel();
	main_widget->krawlRunning(false);
	if (KrawlConfigSkel::showNotifications())
	{
// 		KApplication::kApplication()->lock();
		if (!m_notify_win)
			m_notify_win = new NotifyWindow( main_widget, 0, true, "All Download(s) finished" ); 
		m_notify_win->show();
// 		KApplication::kApplication()->unlock();
	}
	emit completed();
}

void KrawlSitePart::showSettingsDlg()
{
	if (config_dlg == 0)
	{
		config_dlg = new KrawlConfigHandler(main_widget);
		connect(config_dlg->getClearUrlsBtnHandle(), SIGNAL(clicked()), this, SLOT(slotDeleteUrlHistory()));
		connect(config_dlg, SIGNAL(applyClicked()), this, SLOT(config_btnApplyHandler()));
		connect(config_dlg, SIGNAL(okClicked()), this, SLOT(config_btnOkHandler()));
		connect(config_dlg, SIGNAL(defaultClicked()), this, SLOT(config_btnDefaultHandler()));
	}
	config_dlg->show();
}

void KrawlSitePart::clearUrlBar()
{
// 	m_UrlListCombo->setCurrentText("");
}

void KrawlSitePart::config_btnApplyHandler()
{
	config_btnOkHandler(true);
}

void KrawlSitePart::config_btnOkHandler(bool _apply)
{
	config_dlg->slotApplySettings();
// 	if (m_stopAction->isEnabled() == false)  //-----------
	if (main_widget->m_multi_job_handler->isStopBtnEnabled() == false)  //-----------
	{
		main_widget->saveSettings();
		main_widget->loadSavedDefaults(_apply);
		saveSettings();
		loadSettings();
	}
}

void KrawlSitePart::config_btnDefaultHandler()
{
	config_dlg->slotSetDefaults();
}

void KrawlSitePart::slotDeleteUrlHistory()
{
// 	m_UrlListCombo->clear();
	KrawlConfigSkel::setSavedUrls(QStringList());
}

void KrawlSitePart::stopCrawling()
{
	m_stopAction->setEnabled(false); //--------
	main_widget->m_multi_job_handler->setStopBtnState( false );
	emit sigStopCrawling();
}

void KrawlSitePart::slotProgress(KIO::Job *, unsigned long percent)
{
	main_widget->progressUpdate((int)percent);
}

void KrawlSitePart::reportProcessedSize(unsigned long file_size)
{
	m_i_dl_total_size += file_size;
	main_widget->setTotalDlSize(m_i_dl_total_size);
}

void KrawlSitePart::reportCurrentDlSpeed(KIO::Job *, unsigned long speed)
{
	main_widget->setDlSpeed(speed);
}

void KrawlSitePart::slotErrorMesg(QString _err_mesg)
{
	emit setStatusBarText(_err_mesg);
	if (_err_mesg.isEmpty() == false)
		m_err_log += (_err_mesg + "\n");
	if (m_logBrowser != 0 && m_log_win != 0 && m_log_win->isShown())
	{
		if (_err_mesg.isEmpty() == false)
		{
			m_logBrowser->clearLogContents();
		}
		else 
		{
			m_logBrowser->setLogContents(_err_mesg, true);
		}
	}
}

void KrawlSitePart::setStatusText(QString _status_txt)
{
	main_widget->setStatusMsg(_status_txt);
}

void KrawlSitePart::setStatusText(KIO::Job*, const QString& _status_txt)
{
	setStatusText(_status_txt);
}

void KrawlSitePart::slotCreateArchive()
{
	m_ArchiveWidget = new KrawlArchiveWidget(main_widget, main_widget->branchUrl());
	if (m_archive_win == 0)
	{
		m_archive_win = new KDialogBase(KDialogBase::Swallow, Qt::WType_TopLevel, main_widget, "archive_win", false, "Web Archive Creation", KDialogBase::Close, KDialogBase::Close, true);
		m_archive_win->setMainWidget(m_ArchiveWidget);
	}
	m_archive_win->show();		
	m_ArchiveWidget->startArchiveCreation();
}

void KrawlSitePart::slotShowLogWindow()
{
	if (m_logBrowser == 0)
	{
		m_logBrowser = new LogBrowser();
	}
	if (m_log_win == 0)
	{
		m_log_win = new KDialogBase(KDialogBase::Swallow, 0, main_widget, "log_win",  false, "Log-Errors", KDialogBase::Close, KDialogBase::Close, true);
		m_log_win->setMainWidget(m_logBrowser);
	}
	m_logBrowser->setLogContents(m_err_log);
	m_log_win->show();
}

void KrawlSitePart::multiJobHandler()
{
	QStringList combo_url_list = main_widget->getComboUrls();
	killActiveThread();
	m_url = KURL::KURL();
	startCrawling(combo_url_list);
}

void KrawlSitePart::nextJobIndicator(int index, const KURL& _url)
{
	main_widget->updateComboIndex(index);
	openURL(_url);
}

void KrawlSitePart::setLeechFlagTrue() //-- remove this fn
{
// 	main_widget->doneLeech();
}

void KrawlSitePart::setLeechFlagFalse()
{
// 	main_widget->setLeechedFlag(false); -- remove this fn
}

void KrawlSitePart::leechHandler()
{
	t_LeechJobList checked_url_list = main_widget->getCheckedUrls();
	if (checked_url_list.size() == 0)
		return;	
	killActiveThread();
	QString init_dir_str = main_widget->baseDlDirectory();
	m_krawler = new Krawler(this, main_widget, checked_url_list, init_dir_str);
	makeConnections();
	m_krawler->start();
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///// Misc functions/handlers
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void KrawlSitePart::startCrawling()
{
	QStringList dummy;
	startCrawling(dummy);
}

void KrawlSitePart::startCrawling(QStringList /*_comboUrlList*/)
{
	m_stopAction->setEnabled(true); //----------------
	main_widget->m_multi_job_handler->setStopBtnState( true );
	QString qstr_init_dir = main_widget->baseDlDirectory();
	if (qstr_init_dir == 0 || qstr_init_dir.length() == 0)
		return ;
// 	int iMaxLevel = main_widget->crawlDepth();
// 	bool bTraverseOut = main_widget->traverseOut();
// 	bool bLinkOutsidePath = main_widget->linkOutsidePath();
	int krawl_mode = main_widget->krawlMode();
// 	QStringList list_ExcludedFileTypes;
// 	bool fileType_only = main_widget->fileTypeList(list_ExcludedFileTypes);
// 	QStringList dcFileTypes = main_widget->dcTypeList();
	t_KrawlJobList job_list = main_widget->jobMap();
	main_widget->init_widget();
	main_widget->krawlRunning(true);

	setStatusText(i18n("Starting to crawl..."));	
	m_i_dl_total_size = 0;
	m_err_log = "";
	slotErrorMesg(m_err_log);
	debugMsg("mode from part:" + QString("%1").arg(krawl_mode));
	// crawl, crawl
	m_krawler = new Krawler(this, main_widget, qstr_init_dir, job_list);
	
	makeConnections();
	m_krawler->start();
	emit started(0);
}

void KrawlSitePart::killActiveThread()
{
	// ensure no current thread is running
	if (m_krawler != 0)
	{
		if (m_krawler->running())
		{
			emit sigStopCrawling(); 
			setStatusText("Waiting for current thread to finish...");
			m_krawler->wait();
			setStatusText("Current thread finished.");
		}
		delete m_krawler;
		m_krawler = 0;
	}
}

void KrawlSitePart::insertUrl(KURL /*_url*/)
{
/*	if (_url.isEmpty() == false) 
	{
		QStringList qstr_url_list = comboItems();
		if (m_UrlListCombo->contains(_url.url()) == false)
		{
			qstr_url_list.prepend(_url.url());
			m_UrlListCombo->clear();
			m_UrlListCombo->insertStringList(qstr_url_list);
			m_UrlListCombo->setCurrentItem(0);
		}
		else 
		{
			int index = qstr_url_list.findIndex(_url.url());
			m_UrlListCombo->setCurrentItem(index);
		}
	}*/
	
}

void KrawlSitePart::saveSettings()
{
	saveComboUrls = KrawlConfigSkel::chkUrlPersistance();
	if (saveComboUrls) 
	{
		QStringList existing_urls = KrawlConfigSkel::savedUrls();
		QStringList combo_url_list = comboItems();
		QStringList::Iterator it;
		for(it = combo_url_list.begin(); it != combo_url_list.end(); ++it)
		{
			QString str_url = (*it);
			if (!existing_urls.contains(str_url) && !str_url.isEmpty())
			{
				existing_urls.append(str_url);
			}
		}
		KrawlConfigSkel::setSavedUrls(existing_urls);
	}
}

void KrawlSitePart::loadSettings()
{
	saveComboUrls = KrawlConfigSkel::chkUrlPersistance();
/*	m_UrlListCombo->setMaxCount(KrawlConfigSkel::maxNumUrls());
	if (m_UrlListCombo->count() == 0)
	{
		m_UrlListCombo->insertStringList(KrawlConfigSkel::savedUrls());
		m_UrlListCombo->setCurrentText("");
	}*/
}

void KrawlSitePart::customEvent(QCustomEvent *e)
{
	KrawlerEvent* ke = (KrawlerEvent* )e;
	int event_type = ke->eventType();
// 	debugMsg(QString("event type: %1").arg(event_type));
	switch(event_type)
	{
		case KrawlerEvent::FINISHED :
		{
			KwFinishedEvent *kfe = (KwFinishedEvent *)ke;
			slotFinished(kfe->getData());
			break;
		}
		case KrawlerEvent::INFOMESG :
		{
			KwStatusTextEvent *kst = (KwStatusTextEvent *)ke;
			setStatusText(kst->getJob(), kst->m_str);
			break; 	
		}
		case KrawlerEvent::PARSEDLINK :
		{
			KwParsedLinkEvent *kpe = (KwParsedLinkEvent *)ke;
			slotParsedLinkStats(kpe->m_index, kpe->m_embed_url, kpe->m_level, kpe->m_links, kpe->m_imgs);
			break;
		}
		case KrawlerEvent::PERCENT : 
		{
			KwPercentEvent *kpe = (KwPercentEvent *)ke;
			slotProgress(kpe->getJob(), kpe->getData());
			break;
		}
		case KrawlerEvent::PROCESSED :
		{
			KwProcessedEvent *kpe = (KwProcessedEvent *)ke;
			reportProcessedSize(kpe->getData());
			break;
		}
		case KrawlerEvent::IMGSTATS :
		{
			KwImgStatsEvent* kie = (KwImgStatsEvent *)ke;
			slotSetImgStats(kie->m_index, kie->m_img_list_sz);
			break;
		}
		case KrawlerEvent::LINKSTATS :
		{
			KwLinkStatsEvent* kle = (KwLinkStatsEvent *)ke;
			slotSetLinkStats(kle->m_index, kle->m_link_list_sz);
			break;
		}
		case KrawlerEvent::SPEED :	
		{
			KwSpeedEvent *kse = (KwSpeedEvent *)ke;
			reportCurrentDlSpeed(kse->getJob(), kse->getData());
			break;
		}
		case KrawlerEvent::STATUSTEXT :
		{
			KwStatusTextEvent *kse = (KwStatusTextEvent *)ke;
			setStatusText(kse->m_str);
			break;
		}
		case KrawlerEvent::ERRORMESG :
		{
			KwErrorMesgEvent *kee  = (KwErrorMesgEvent *)ke;
			slotErrorMesg(kee->m_str);
			break;
		}
		case KrawlerEvent::CURRENTURL :
		{
			KwCurrentDlUrlEvent *kce = (KwCurrentDlUrlEvent *)ke;
			slotCurrentDlUrl(kce->m_url_str, kce->m_url_type);
			break;
		}
		case KrawlerEvent::NEXTJOB :
		{
			KwNextJobEvent *kne = (KwNextJobEvent *)ke;
			nextJobIndicator(kne->getData(), kne->m_url);
			break;
		}
		case KrawlerEvent::SELDLCOMPLETE :
		{
			setLeechFlagFalse();
			break;
		}
		case KrawlerEvent::MARKER :
		{
			KwMarkerEvent *kme = (KwMarkerEvent *)ke;
			main_widget->m_url_display->markCurrentDownload(kme->getData(), kme->m_index, kme->m_url_type);
			break;
		}
		default:
		{
			int* data = static_cast<int *>(e->data());
			if (data != 0)
			{
// 				debugMsg(QString("Unhandled custom event:%1").arg(*data));
				main_widget->m_multi_job_handler->leechBarClicked(*data);
			}
			break;
		}
	}
}

void KrawlSitePart::makeConnections()
{
	// make the connections
	connect(main_widget, SIGNAL(jumpToNextLink()), m_krawler, SLOT(gotoNextLink()));
	connect(main_widget, SIGNAL(jumpToNextUrl()), m_krawler, SLOT(gotoNextURL()));
	connect(main_widget, SIGNAL(pauseKrawl(bool )), m_krawler, SLOT(pauseKrawl(bool )));
	connect(main_widget, SIGNAL(ignoreLink(const QString& )), m_krawler, SLOT(slotIgnoreLink(const QString& )));
	connect(this, SIGNAL(sigStopCrawling()),
			m_krawler, SLOT(stopCrawlingClicked()));
}

QStringList KrawlSitePart::comboItems()
{
	return main_widget->m_multi_job_handler->getComboUrls();
}

void KrawlSitePart::debugMsg(QString _msg)
{
	kdDebug(9815578)<<"KrawlSitePart..."<<_msg<<endl;
}

#include "krawlsite_part.moc"


// factory code - no messing around with this :-s
// It's usually safe to leave the factory code alone.. with the
// notable exception of the KAboutData data
#include <kaboutdata.h>

KInstance*  KrawlSitePartFactory::s_instance = 0L;
KAboutData* KrawlSitePartFactory::s_about = 0L;

KrawlSitePartFactory::KrawlSitePartFactory()
    : KParts::Factory()
{
}

KrawlSitePartFactory::~KrawlSitePartFactory()
{
    delete s_instance;
    delete s_about;

    s_instance = 0L;
}

KParts::Part* KrawlSitePartFactory::createPartObject( QWidget *parentWidget, const char *widgetName,
                                                        QObject *parent, const char *name,
                                                        const char *classname, const QStringList &args )
{
    // Create an instance of our Part
    KrawlSitePart* obj = new KrawlSitePart( parentWidget, widgetName, parent, name );

    // See if we are to be read-write or not
//     if (QCString(classname) == "KParts::ReadOnlyPart")
//         obj->setReadWrite(false);

    return obj;
}

KInstance* KrawlSitePartFactory::instance()
{
    if( !s_instance )
    {
        s_about = new KAboutData("krawlsitepart", I18N_NOOP("Krawler"), "0.7");
        s_about->addAuthor("Sayantam Dey", 0, "sdey@ics.uci.edu");
        s_instance = new KInstance(s_about);
    }
    return s_instance;
}

extern "C"
{
    void* init_libkrawlsitepart()
    {
        return new KrawlSitePartFactory;
    }
};
