/***************************************************************************
 *   Copyright (C) 2004 by Sayantam Dey                                    *
 *   sayantam@gmail.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _KRAWLSITEPART_H_
#define _KRAWLSITEPART_H_

#include <kparts/part.h>
#include <kparts/factory.h>
#include <kio/job.h>
#include <kaction.h>
#include <kurlcombobox.h>
#include <kconfig.h>
#include <kdialogbase.h>

#include "krawlmainwidget.h"
#include "krawlconfig.h"
#include "krawler.h"
#include "logbrowser.h"
#include "krawlarchiver.h"
#include "notifiywindow.h"
#include "commonds.h"

class QWidget;
class QPainter;
class KURL;
class QMultiLineEdit;
class QEvent;
/**
 * This is a "Part".  It that does all the real work in a KPart
 * application.
 *
 * @short Main Part
 * @author Sayantam Dey <sdey@ics.uci.edu>
 * @version 0.1
 */

class KrawlSitePart : public KParts::ReadOnlyPart
{
    Q_OBJECT
public:

    KrawlSitePart(QWidget *parentWidget, const char *widgetName,
                    QObject *parent, const char *name);

    virtual ~KrawlSitePart();

public slots:
	virtual bool openURL(const KURL& _url);
	
protected:
    virtual bool openFile();
	void customEvent(QCustomEvent *);
	
protected slots:
	void stopCrawling();
	void goClicked();
	void slotProgress(KIO::Job *, unsigned long);
	void reportProcessedSize(unsigned long);
	void reportCurrentDlSpeed(KIO::Job *, unsigned long);
	void config_btnDefaultHandler();
	void config_btnApplyHandler();
	void config_btnOkHandler(bool _apply = false);
	void slotDeleteUrlHistory();
	void showSettingsDlg();
	void slotFinished(int);
	void slotParsedLinkStats(int, QString, int, const QStringList&, const QStringList& );	
	void slotSetImgStats(int, int );
	void slotSetLinkStats(int, int );
	void setStatusText(QString);
	void setStatusText(KIO::Job*, const QString& );
	void slotErrorMesg(QString);
	void slotCreateArchive();
	void slotShowLogWindow();
	void slotCurrentDlUrl(QString, int);
	void multiJobHandler();
	void killActiveThread();
	void startCrawling();
	void startCrawling(QStringList _comboUrlList);
	void nextJobIndicator(int, const KURL& );
	void setLeechFlagTrue();
	void setLeechFlagFalse();
	void leechHandler();
	void clearUrlBar();
			
private:
    KrawlMainWidget *main_widget;
	KrawlConfigHandler* config_dlg;
	Krawler *m_krawler;
	KDialogBase *m_log_win;
	LogBrowser* m_logBrowser;
	KrawlArchiveWidget* m_ArchiveWidget;
	KDialogBase* m_archive_win;
	NotifyWindow* m_notify_win;
	
protected: //methods
	void insertUrl(KURL _url);
	void saveSettings();
	void loadSettings();
	void makeConnections();
	QStringList comboItems();
	void debugMsg(QString _str);
		
protected: // attributes
	unsigned long m_i_dl_total_size;
	bool saveComboUrls;	
	QString m_err_log;
	
	//gui attributes
// 	KURLComboBox* m_UrlListCombo;
// 	KWidgetAction* combo_action;
	KAction* m_goAction;
	KAction* m_stopAction;
	KAction* m_startAction;
	KAction* m_settingsAction;
	KAction* m_archiveAction;
	KAction* m_showLogWin;
// 	KAction* m_clearLocationBar;
	KIO::Job* m_current_kio_job;

signals:
	void sigStopCrawling();		
};

class KInstance;
class KAboutData;

class KrawlSitePartFactory : public KParts::Factory
{
    Q_OBJECT
public:
    KrawlSitePartFactory();
    virtual ~KrawlSitePartFactory();
    virtual KParts::Part* createPartObject( QWidget *parentWidget, const char *widgetName,
                                            QObject *parent, const char *name,
                                            const char *classname, const QStringList &args );
    static KInstance* instance();
 
private:
    static KInstance* s_instance;
    static KAboutData* s_about;
};

#endif // _KRAWLSITEPART_H_
