/***************************************************************************
 *   Copyright (C) 2004 by Sayantam Dey                                    *
 *   sayantam@gmail.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef MULTIJOBHANDLER_H
#define MULTIJOBHANDLER_H

#include <kcombobox.h>
#include <ktoolbar.h>
#include <kdialogbase.h>

#include <qwidget.h>

#include "krawler.h"
#include "dropwindow.h"
#include "quicksetoptions.h"
#include "infopanel.h"
#include "historybrowser.h"

/**
@author Sayantam Dey
*/
class MultiJobHandler : public QWidget
{
Q_OBJECT
public:
    MultiJobHandler(QWidget *parent = 0, const char *name = 0, WId _container_id = -1);
    ~MultiJobHandler();
	
	QStringList getComboUrls();
	QString updateComboIndex(int index);
	void krawlRunning(bool _flag);
	void setInfoPanelInst(InfoPanel* _info_panel_inst);
	void setPageOptions(QuickSetOptions* _qso_frame);
	void setOpMode(int _mode);
	void init_widget(int _mode);
	void enableSave(bool _flag);
	void setStopBtnState(bool _enable);
	bool isStopBtnEnabled();
	
	enum OpMode
	{
		SINGLE_JOB = 1, MULTI_JOB
	};
	t_KrawlJobList m_KrawlJobList;

public slots:	
	void dropWindowHandler(QString);
	void leechBarClicked(int);
	
protected slots:
	void dropWindowHandler(KURL::List);
	void dropWindowClosed(const int, const int);
	void cmbReturnPressed(const QString&);
	void urlSettingsOkClicked();
	void btnDropTargetClicked();
	void slotAddUrlFromHistory(QString);
	void slotAddUrlFromHistory();
	void cmbTextChanged(const QString &);
		
protected:
	enum leechbar_ids
	{
		L_START = 8, L_NEXT_URL, L_NEXT_LINK, L_PAUSE, L_REMOVE, L_CLEAR, L_SAVE, L_SEP, L_OPTIONS, L_COMBO, L_DROP, L_STOP, L_TAR, L_LOG, L_CONF,
		L_HIST, L_ADD
	};
	
	KToolBar* m_LeechBar; 
	DropWindow* win;	
	KDialogBase* m_dlgJobSettings;
    KToolBarButton* m_btnDropTarget;
    KComboBox* m_cmbUrlsList;
	QuickSetOptions* qso_frame;
	QuickSetOptions* m_qso_holder;
	int drop_win_x, drop_win_y;
	bool krawl_paused;
	InfoPanel* m_info_panel_inst;
	int m_op_mode;
	WId m_container_id;
	HistoryBrowser* m_hist_br;
	KDialogBase* m_history_dlg;	
	bool m_drop_target_flag;
	
	//functions
	void createLeechBar();
	void debugMsg(QString _str);
	t_KrawlJob createKrawlJob(QString _url_str);
	void addKrawlJobs(QStringList _url_str_list);
	
signals:
	void startMultiJob();
	void startLeech();
	void jumpToNextLink();
	void jumpToNextUrl();
	void pauseKrawl(bool);
	void createArchive();
	void showLog();
	void showConfigure();
	void stopCrawling();
};

#endif
