/***************************************************************************
 *   Copyright (C) 2004 by Sayantam Dey                                    *
 *   sayantam@gmail.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <kglobalsettings.h>
#include <klocale.h>
 
#include <qpainter.h>
#include <qbitmap.h>
#include <qapplication.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qtimer.h>

#include "notifiywindow.h"
#include "krawlconfig.h"

NotifyWindow::NotifyWindow(QWidget *parent, const char *name, bool _enable_timer, const char* _disp_text)
 : QWidget(parent, name, WType_TopLevel | WNoAutoErase | WStyle_Customize | WX11BypassWM | WStyle_StaysOnTop),
 	default_width(150), default_height(20)
{
    if (!name)
		setName("notify window");
	setFocusPolicy( NoFocus );
	setBackgroundMode( NoBackground );
	drawWindow(_disp_text);
	
	pm->setMask(pm->createHeuristicMask(true));
 	mask = *pm->mask();
	setBackgroundPixmap(*pm);
	setMask(mask);
	
	m_enable_timer = _enable_timer;
	resize( QSize(default_width, default_height).expandedTo(minimumSizeHint()) );
	
	repositionWidget();
}

NotifyWindow::~NotifyWindow()
{
}

void NotifyWindow::drawWindow(QString _disp_text)
{
	QFontMetrics fm( KGlobalSettings::windowTitleFont() );
	QRect bounding_rect = fm.boundingRect(_disp_text);
	default_width = bounding_rect.width() + 30;
	default_height = bounding_rect.height() + 20;
	// thanks ~ amarok team :)
	uint char_width = fm.width('x'); 
	uint round_x = (char_width * 200)/bounding_rect.width();
	uint round_y = (char_width * 200)/bounding_rect.height();
	int text_width = bounding_rect.width() + 10;
	int text_height = bounding_rect.height() + 10;
	//
	pm = new QPixmap( QSize( default_width, default_height ) );
	pm->fill( Qt::black );
	QPainter p;
	p.begin(pm);
	p.setPen( Qt::black ); 
	p.setBrush( KGlobalSettings::highlightColor() );
	p.drawRoundRect( 0, 0, default_width, default_height, round_x, round_y);
	p.setFont( KGlobalSettings::windowTitleFont() );
	p.setPen( KGlobalSettings::activeTextColor() );
	int sx = (default_width - text_width)/2;
	int sy = (default_height - text_height)/2;
	p.drawText( sx, sy, text_width, text_height, Qt::AlignCenter, _disp_text );
	p.end();
}

void NotifyWindow::timerSlot()
{
	hide();
}

void NotifyWindow::mousePressEvent(QMouseEvent* e)
{
	if (e->button() == LeftButton)
	{
		if (m_enable_timer)
		{
			hide();
		}
		else oldX = oldY = 0;
	}
}

void NotifyWindow::mouseMoveEvent(QMouseEvent* e)
{
	if (oldX == 0)
	{
		oldX = e->x();
		oldY = e->y();
		return;
	}
	move(x() + (e->x() - oldX), 	y() + (e->y() - oldY));
}

void NotifyWindow::repositionWidget()
{
	QPoint p = KrawlConfigSkel::notifyWinPos();
	int desktop_width = QApplication::desktop()->width();
	int desktop_height = QApplication::desktop()->height();
	if ( ! (p.x() >= 0 && p.x() <= desktop_width && p.y() >= 0 && p.y() <= desktop_height) )
	{
		int x = (desktop_width - default_width) / 2;
		int y = (desktop_height - default_height) / 2;
		p = QPoint::QPoint(x,y);
	}
	oldX = p.x();
	oldY = p.y();
	move(p);
}

void NotifyWindow::showEvent(QShowEvent* )
{
	if (m_enable_timer)
	{
		QTimer::singleShot(5000, this, SLOT(timerSlot()));
	}
}
#include "notifiywindow.moc"
