/***************************************************************************
 *   Copyright (C) 2004 by Sayantam Dey                                    *
 *   sayantam@gmail.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <knuminput.h>
#include <klineedit.h>
#include <kdebug.h>

#include <qbuttongroup.h>
#include <qcheckbox.h>
#include <qradiobutton.h>

#include "krawler.h"
#include "krawlconfig.h"
#include "quicksetoptions.h"

QuickSetOptions::QuickSetOptions(QWidget* parent, const char* name, WFlags fl)
: PgOptions(parent,name,fl)
{
	connect(m_grpMode, SIGNAL(clicked(int)), this, SLOT(krawlModeChanged(int)));
	setMinimumSize(sizeHint());
}

QuickSetOptions::~QuickSetOptions()
{
	kdDebug(9815578)<<"QuickSetOptions... destructor called"<<endl;
}

int QuickSetOptions::crawlDepth()
{
	return m_numCrawlDepth->value();
}

bool QuickSetOptions::traverseOut()
{
	return m_chkTraverseOut->isChecked();
}

bool QuickSetOptions::linkOutsidePath()
{
	return m_chkExternalPath->isChecked();
}

void QuickSetOptions::loadSettings()
{
	m_numCrawlDepth->setValue(KrawlConfigSkel::defCrawlDepth());
	m_chkTraverseOut->setChecked(KrawlConfigSkel::chkOutsideDomain());
	m_chkExternalPath->setChecked(KrawlConfigSkel::chkOutsidePath());
	krawlModeChanged();
	m_rbtnInclude->setChecked(KrawlConfigSkel::fileIncMode());
	m_rbtnExclude->setChecked(KrawlConfigSkel::fileExMode());
	m_txtTypes->setText(KrawlConfigSkel::fileTypes().join(","));
	m_txtDynContent->setText(KrawlConfigSkel::dcFileTypes().join(","));
}

void QuickSetOptions::krawlModeChanged(int _button_id)
{
	if (_button_id == -1)
	{
		m_rbtnSimple->setChecked(KrawlConfigSkel::simpleCrawl());
		m_rbtnOffline->setChecked(KrawlConfigSkel::offlineBrowse());
		m_rbtnLeech->setChecked(KrawlConfigSkel::leech());
	}
	setKrawlMode(krawlMode());
}

void QuickSetOptions::setKrawlMode(int _krawl_mode)
{
	emit changedKrawlMode(modeDescStr(_krawl_mode));
}

QString QuickSetOptions::modeDescStr(int _krawl_mode)
{
	QString mode_str;
	
	switch(_krawl_mode)
	{
		case Krawler::Default:
		{
			mode_str = "Simple Crawler";
			break;
		}
		case Krawler::Offline_Browser: 
		{
			mode_str = "Offline Browser";		
			break;
		}
		case Krawler::Leech:
		{
			mode_str = "Leech Agent";
			break;
		}
		default:
		{
			mode_str = "Simple Crawler";
		}
	}
	
	return mode_str;	
}

int QuickSetOptions::krawlMode()
{
	int mode = -1;
	if (m_rbtnSimple->isChecked())
	{
		mode = Krawler::Default;
	}
	else if (m_rbtnOffline->isChecked())
	{
		mode = Krawler::Offline_Browser;
	}
	else if (m_rbtnLeech->isChecked())
	{
		mode = Krawler::Leech;
	}
	return mode;
}

bool QuickSetOptions::fileTypeList(QStringList &_type_list)
{
	_type_list = QStringList::split(',', m_txtTypes->text());
	return m_rbtnInclude->isChecked();
}

QStringList QuickSetOptions::dcTypeList()
{
	return QStringList::split(',', m_txtDynContent->text());
}

JobOptions& QuickSetOptions::jobOptions(JobOptions& job_opt)
{
	if (job_opt.set_flag == false)
		job_opt.set_flag = true;
	job_opt.crawl_mode = krawlMode();
	job_opt.crawl_depth = m_numCrawlDepth->value();
	job_opt.linkOutsidePath = m_chkExternalPath->isChecked();
	job_opt.traverseOut = m_chkTraverseOut->isChecked();
	job_opt.exclusive_flag = !fileTypeList(job_opt.fileTypeList);
	job_opt.dcTypes = dcTypeList();
	
	return job_opt;
}

void QuickSetOptions::loadSettingsFromOptions(const JobOptions& job_opt)
{
	switch(job_opt.crawl_mode)
	{
		case Krawler::Offline_Browser:
			m_rbtnOffline->setChecked(true);
			break;
		case Krawler::Leech:
			m_rbtnLeech->setChecked(true);
			break;
		default:
			m_rbtnSimple->setChecked(true);
	}
	m_numCrawlDepth->setValue(job_opt.crawl_depth);
	m_chkExternalPath->setChecked(job_opt.linkOutsidePath);
	m_chkTraverseOut->setChecked(job_opt.traverseOut);
	m_rbtnExclude->setChecked(job_opt.exclusive_flag);
	m_rbtnInclude->setChecked(!job_opt.exclusive_flag);
	m_txtTypes->setText(job_opt.fileTypeList.join(","));
	m_txtDynContent->setText(job_opt.dcTypes.join(","));		
	internal_options_object = job_opt;
}

JobOptions& QuickSetOptions::internalOptionsObject()
{
	return internal_options_object;
}
/*$SPECIALIZATION$*/

#include "quicksetoptions.moc"

