/***************************************************************************
 *   Copyright (C) 2004 by Sayantam Dey                                    *
 *   sayantam@gmail.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <kdebug.h>
#include <klocale.h>
#include <kglobal.h> 
#include <kglobalsettings.h>
#include <kiconloader.h>
#include <krun.h>
#include <kio/netaccess.h>
#include <kmessagebox.h>
#include <kapplication.h>
#include <qfileinfo.h>

#include <qdragobject.h>
#include <qapplication.h>
#include <qclipboard.h>
#include <qlabel.h>
#include <qpointarray.h>

#include "krawlconfig.h"
#include "urldisplay.h"
#include "urldisplay.moc"

UrlDisplay::UrlDisplay(const int _initial_width, QWidget *parent, const char *name)
 : QWidget(parent, name)
{
	m_klvParsedUrls = new ParsedUrlListView(this, "parsed_urls", _initial_width);
	m_klvParsedUrls->setDragEnabled(true);
	connect(m_klvParsedUrls, SIGNAL(contextMenu(KListView*, QListViewItem*, const QPoint& )),
				this, SLOT(slotUrlListContextMenu(KListView*, QListViewItem*, const QPoint&)));
	connect(m_klvParsedUrls, SIGNAL(dropped(QDropEvent *, QListViewItem *, QListViewItem *)),
				this, SLOT(urlDropped(QDropEvent *)));
	connect(m_klvParsedUrls, SIGNAL(selectionChanged()), this, SLOT(slotItemSelectionChanged()));
	connect(m_klvParsedUrls, SIGNAL(clicked(QListViewItem* )), this, SLOT(slotUrlViewClicked()));
	q_layout = new QVBoxLayout(this, 0, 0, "v-layout");
	q_layout->addWidget(m_klvParsedUrls);
	m_image_pix = QPixmap(KGlobal::iconLoader()->loadIcon("image", KIcon::NoGroup, 16));
	m_html_pix = QPixmap(KGlobal::iconLoader()->loadIcon("html", KIcon::NoGroup, 16));
	url_list_menu_ptr = new KPopupMenu(this, "url_popup_menu");
	///
	m_mark_pix = QPixmap(QSize(16,16));
	m_mark_pix.fill(m_klvParsedUrls->backgroundColor());
	qApp->lock();
	QPainter p;
	p.begin(&m_mark_pix);
	p.setPen(KGlobalSettings::highlightedTextColor());
	p.setBrush(KGlobalSettings::highlightColor());
	QPointArray p_arr(3);
	p_arr.putPoints(0, 3, 0, 0, 16, 8, 0, 16);
	p.drawPolygon(p_arr, false, 0, 3);
	p.end();
	qApp->unlock();
	///
	setSizePolicy( QSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding, 0, 0, sizePolicy().hasHeightForWidth() ) );
	m_last_marked = 0;
}


UrlDisplay::~UrlDisplay()
{
// 	delete image_pix;
// 	image_pix = 0;
// 	delete html_pix;
// 	html_pix = 0;
}

void UrlDisplay::addLinkInfo(int _index, QString q_str_url, int link_depth, const QStringList& _links, const QStringList& _images)
{
	debugMsg("addLinkInfo called");
	t_KrawlJob krawl_job = m_mjh_inst->m_KrawlJobList[_index];
	JobOptions job_opt = krawl_job.job_options;
	int mode = m_current_mode = job_opt.crawl_mode;
	
	//qApp->lock();
	// close the previous item
	if (m_lastItem != 0)
		m_lastItem->setOpen(false);
	
	//create root item struct
	t_UrlRootItem root_item;
	root_item.m_index = _index;
	// add the link
	if (mode != Krawler::Leech)
	{
		m_lastItem = new KListViewItem(	m_klvParsedUrls,	
										m_lastItem,	
										q_str_url, 
										QString("%1").arg(link_depth), 
										QString("%1").arg((int) _links.size()), 
										QString("%1").arg((int) _images.size()));
	}
	else
	{
		m_lastItem = new QCheckListItem(m_klvParsedUrls, m_lastItem, q_str_url, QCheckListItem::CheckBoxController);
		m_lastItem->setText(1, QString("%1").arg(link_depth));
		m_lastItem->setText(2, QString("%1").arg((int) _links.size()));
		m_lastItem->setText(3, QString("%1").arg((int) _images.size()));
	}
	m_lastItem->setOpen(true);
	root_item.m_item_ptr = m_lastItem;
	
	// add the embedded links
	QStringList::const_iterator it;
	uint index = 0;
	
	//images
// 	root_item.m_vec_ImgUrl = new QValueVector<t_UrlChildItem>(_images.size());
	for (it = _images.begin(); it != _images.end(); it++)
	{
		QString qstr_link = (*it);
		if (mode != Krawler::Leech)
		{
			m_lastChild = new KListViewItem(m_lastItem, m_lastChild, qstr_link, QString::null, QString::null);
		}	
		else 
		{
			m_lastChild = new QCheckListItem(m_lastItem, m_lastChild, qstr_link, QCheckListItem::CheckBox);
			m_lastChild->setText(1, QString::null);
			m_lastChild->setText(2, QString::null);
		}
		m_lastChild->setPixmap(0, m_image_pix);
		t_UrlChildItem child_item;
		child_item.m_index = index;
		child_item.m_item_ptr = m_lastChild;
		root_item.m_vec_ImgUrl.append(child_item);
		root_item.m_vec_ImgUrl[index] = child_item;
		index++;
	}
	
	//links
	index = 0;
// 	root_item.m_vec_HrefUrl = new QValueVector<t_UrlChildItem>(_links.size());
	for (it = _links.begin(); it != _links.end(); it++)
	{
		QString qstr_link = (*it);
		if (mode != Krawler::Leech)
		{
			m_lastChild = new KListViewItem(m_lastItem,
											m_lastChild,
											qstr_link);
		}
		else 
		{
			m_lastChild = new QCheckListItem(m_lastItem, m_lastChild, qstr_link, QCheckListItem::CheckBox);
		}
		m_lastChild->setPixmap(0, m_html_pix);
		t_UrlChildItem child_item;
		child_item.m_index = index;
		child_item.m_item_ptr = m_lastChild;
		root_item.m_vec_HrefUrl.append(child_item);
		root_item.m_vec_HrefUrl[index] = child_item;
		index++;
	}
	//qApp->unlock();
	m_vec_url_root_items.append(root_item);
	m_vec_url_root_items[m_vec_url_root_items.count()-1] = root_item;
	
	m_info_panel_inst->setLinkPrgsTotal(_links.size());
	m_info_panel_inst->setImagePrgsTotal(_images.size());
}

void UrlDisplay::init_widget()
{
	if (m_last_marked != 0)
	{
		if (m_last_marked_pix_type == Krawler::IMG)
		{
			m_last_marked->setPixmap(0, m_image_pix);
		}
		else m_last_marked->setPixmap(0, m_html_pix);
	}
	m_klvParsedUrls->clear();
	m_vec_url_root_items.clear();
	m_lastItem = 0;
	m_lastChild = 0;
	m_last_marked = 0;
	m_current_mode = -1;
}

KPopupMenu* UrlDisplay::createUrlViewPopupMenu(KPopupMenu* _popup_menu_ptr)
{
	int index = 0;
	_popup_menu_ptr->clear();
	_popup_menu_ptr->insertTitle(i18n("URL View"), index++);
	_popup_menu_ptr->insertItem(QIconSet(KGlobal::iconLoader()->loadIcon("editcopy", KIcon::NoGroup, 16)), i18n("&Copy URL(s)"), ID_COPY_URL, index++);
	_popup_menu_ptr->insertSeparator(index++);
	_popup_menu_ptr->insertItem(i18n("&View Source"), ID_VIEW_SRC, index++);
	_popup_menu_ptr->insertSeparator(index++);
	_popup_menu_ptr->insertItem(i18n("&Toggle Check URL(s)"), ID_SELECT_URL, index++);
	_popup_menu_ptr->insertItem(i18n("&Ignore URL(s)"), ID_IGNORE_URL, index++);
	return _popup_menu_ptr;	
}

void UrlDisplay::markCurrentDownload(QString str_url, int _url_type)
{
/*	QListViewItem* item = m_klvParsedUrls->currentDownloadItem(m_lastItem, str_url);
	if (item != 0)
	{
		//qApp->lock();
		if (m_last_marked != 0)
		{
			if (m_last_marked_pix_type == Krawler::IMG)
			{
				m_last_marked->setPixmap(0, *image_pix);
			}
			else m_last_marked->setPixmap(0, *html_pix);
		}
		m_klvParsedUrls->ensureItemVisible(item);
		m_last_marked_pix_type = _url_type;
		item->setPixmap(0, *mark_pix);
		m_last_marked = item;
		//qApp->unlock();
	}*/
}

void UrlDisplay::markCurrentDownload(QListViewItem* _item, int _url_type)
{
	debugMsg("markCurrentDownload called"); 

	if (_item != 0)
	{
		if (m_last_marked != 0)
		{
			if (m_last_marked_pix_type == Krawler::IMG)
			{
				m_last_marked->setPixmap(0, m_image_pix);
			}
			else m_last_marked->setPixmap(0, m_html_pix);
		}
		m_klvParsedUrls->ensureItemVisible(_item);
		m_last_marked_pix_type = _url_type;
		_item->setPixmap(0, m_mark_pix);
		m_last_marked = _item;
	}
}

void UrlDisplay::markCurrentDownload(uint _root_index, uint _child_index, int _link_type)
{
	if ( _root_index < m_vec_url_root_items.size() )
	{
		t_UrlRootItem root_item = m_vec_url_root_items[_root_index];
		QListViewItem *to_be_marked = 0;
		if (m_current_mode != Krawler::Leech)
		{
			--_child_index;
		}
		if ( _link_type == Krawler::IMG )
		{
			if ( _child_index < root_item.m_vec_ImgUrl.size() )
			{
				t_UrlChildItem child_item = root_item.m_vec_ImgUrl[_child_index];
				to_be_marked = child_item.m_item_ptr;
			}
			else
			{
				debugMsg("error! _child_index outta range!");
				return;
			}
		}
		else
		{
			if ( _child_index < root_item.m_vec_HrefUrl.size() )
			{
				t_UrlChildItem child_item = root_item.m_vec_HrefUrl[_child_index];
				to_be_marked = child_item.m_item_ptr;
			}
			else
			{
				debugMsg("error! _child_index outta range!");
				return;
			}
		}
		markCurrentDownload(to_be_marked, _link_type);	
	}
	else
	{
		debugMsg("error! _root_index outta range!");
		return;
	}
}

void UrlDisplay::urlDropped(QDropEvent* e)
{
	e->accept(true);
	QString url_str;
	if (QTextDrag::decode(e, url_str))
	{
		m_mjh_inst->dropWindowHandler( url_str );
	}
}

void UrlDisplay::slotCopyUrl(int id, QString _sel_url_str)
{
	switch(id)
	{
		case ID_COPY_URL:
		{
			QClipboard* qcb = QApplication::clipboard();
			qcb->setText(_sel_url_str, QClipboard::Clipboard);
			break;
		}
		case ID_VIEW_SRC:
		{
			QString tmpFile;
			if (KIO::NetAccess::download(KURL::KURL(_sel_url_str), tmpFile, this))
			{
				KRun::runURL(KURL::KURL(tmpFile), "text/plain", true);
			}
			break;
		}
		case ID_IGNORE_URL:
		{
			emit ignoreLink(_sel_url_str);
			break;
		}
		case ID_SELECT_URL:
		{
			if (m_qso_inst->krawlMode() == Krawler::Leech)
			{
				QPtrList<QListViewItem> selected_items = m_klvParsedUrls->selectedItems();
				QListViewItem* ite = 0;
				QCheckListItem* chk_ite = 0;
				for ( ite = selected_items.first(); ite; ite = selected_items.next() )
				{
					chk_ite = (QCheckListItem*) ite;
					if ( !chk_ite->isOn() )
					{
						chk_ite->setState(QCheckListItem::On);
					}
					else chk_ite->setState(QCheckListItem::Off);
				}
				if ( isCheckedItems() )
				{
					m_mjh_inst->enableSave(true);
				}
				else m_mjh_inst->enableSave(false);
			}
			break;
		}
		default:
			break;
	}
}

void UrlDisplay::slotUrlListContextMenu(KListView*, QListViewItem* _qlvi, const QPoint& p)
{
	if (_qlvi == 0)
		return;
	url_list_menu_ptr = createUrlViewPopupMenu(url_list_menu_ptr);
	int id = url_list_menu_ptr->exec(p);
	
	QPtrList<QListViewItem> selected_items = m_klvParsedUrls->selectedItems();
	if (selected_items.count() < 1)
		return;
	if (selected_items.count() == 1)
	{
		QString sel_url_str = _qlvi->text(0);
		slotCopyUrl(id, sel_url_str);
		return;
	}
	if (selected_items.count() > 1)
	{
		if (id == ID_VIEW_SRC)
		{
			QString sel_url_str = _qlvi->text(0);
			slotCopyUrl(id, sel_url_str);
			return;
		}
		QStringList selected_url_list;
		for ( QListViewItem* list_ite = selected_items.first(); list_ite; list_ite = selected_items.next() )
			selected_url_list.append(list_ite->text(0));
		slotCopyUrl(id, selected_url_list.join(","));
	}
}

void UrlDisplay::setInfoPanelInst(InfoPanel* _info_panel_inst)
{
	m_info_panel_inst = _info_panel_inst;
}

void UrlDisplay::setPageOptions(QuickSetOptions* _qso_options)
{
	m_qso_inst = _qso_options;
}

void UrlDisplay::setMultiJobHandlerInst(MultiJobHandler* _mjh_inst)
{
	m_mjh_inst = _mjh_inst;
}

bool UrlDisplay::isCheckedItems()
{
	bool flag = false;
	QListViewItemIterator it(m_klvParsedUrls, QListViewItemIterator::Checked);
	while ( it.current() )
	{
		flag = true;
		break;
	}
	
	return flag;
}

t_LeechJobList UrlDisplay::getCheckedUrls()
{
	t_LeechJobList m_LeechJobList;
	for ( uint i = 0; i < m_vec_url_root_items.size(); i++ )
	{
		t_UrlRootItem root_item = m_vec_url_root_items[i];
		t_LeechJob leech_job;
		leech_job.m_index = root_item.m_index;
		if (m_mjh_inst->m_KrawlJobList[i].job_options.crawl_mode == Krawler::Leech)
		{ 
			m_current_mode = Krawler::Leech;
			for ( uint j = 0; j < root_item.m_vec_ImgUrl.size(); j++ )
			{
				t_UrlChildItem child_item = root_item.m_vec_ImgUrl[j];
				QCheckListItem* child_check_item = (QCheckListItem* ) child_item.m_item_ptr;
				if ( child_check_item->isOn() )
				{
					t_LeechUrl leech_url;
					leech_url.m_index = child_item.m_index;
					leech_url.m_str_url = child_item.m_item_ptr->text(0);
					leech_url.m_url_type = Krawler::IMG;
					leech_job.m_LeechUrlList.append( leech_url );
				}
			}
			for ( uint k = 0; k < root_item.m_vec_HrefUrl.size(); k++ )
			{
				t_UrlChildItem child_item = root_item.m_vec_HrefUrl[k];
				QCheckListItem* child_check_item = (QCheckListItem* ) child_item.m_item_ptr;
				if ( child_check_item->isOn() )
				{
					t_LeechUrl leech_url;
					leech_url.m_index = child_item.m_index;
					leech_url.m_str_url = child_item.m_item_ptr->text(0);
					leech_url.m_url_type = Krawler::HREF;
					leech_job.m_LeechUrlList.append( leech_url );
				}
			}
			if ( leech_job.m_LeechUrlList.size() > 0 )
				m_LeechJobList.append( leech_job ); 
		}
	}
	return m_LeechJobList;	
}

void UrlDisplay::slotItemSelectionChanged()
{
	if ( m_qso_inst->krawlMode() == Krawler::Leech && KrawlConfigSkel::chkSelectChecks() )
	{
		// uncheck the root item to uncheck all first
		QCheckListItem* root_item = (QCheckListItem *) m_klvParsedUrls->firstChild();
		root_item->setOn(false);
		QPtrList<QListViewItem> selected_items = m_klvParsedUrls->selectedItems();
		for ( QListViewItem* ite = selected_items.first(); ite ; ite = selected_items.next() )
			((QCheckListItem*) ite)->setOn(true);
		
		if ( isCheckedItems() )
		{
			m_mjh_inst->enableSave(true);
		}
		else m_mjh_inst->enableSave(false);
	}	
}

void UrlDisplay::slotUrlViewClicked()
{
	if ( m_qso_inst->krawlMode() == Krawler::Leech && !KrawlConfigSkel::chkSelectChecks() )
	{
		if ( isCheckedItems() )
		{
			m_mjh_inst->enableSave(true);
		}
		else m_mjh_inst->enableSave(false);
	}	
}

void UrlDisplay::debugMsg(QString _str)
{
// 	kdDebug(9815578)<<"UrlDisplay.... "<<_str<<endl;
}

ParsedUrlListView::ParsedUrlListView(QWidget* _parent, const char* _name, int _splitter_width)
	: KListView(_parent, _name)
{
// 	kdDebug(9815578)<<QString("splitter width: %1").arg(_splitter_width)<<endl;
	addColumn(tr("Parsed URL"), (3*_splitter_width)/4);
	addColumn(tr("Link Depth"));
	addColumn(tr("# Links"));
	addColumn(tr("# Images"));
	setRootIsDecorated(true);
	setSorting(-1);
	setColumnAlignment(1, Qt::AlignHCenter);	
	setColumnAlignment(2, Qt::AlignHCenter);	
	setColumnAlignment(3, Qt::AlignHCenter);	
	setAlternateBackground(KGlobalSettings::alternateBackgroundColor());
	setAcceptDrops(true);
 	setDropVisualizer(true);
	setSelectionMode(QListView::Extended);
}

ParsedUrlListView::~ParsedUrlListView()
{
}

bool ParsedUrlListView::acceptDrag(QDropEvent* _event) const
{
	return QTextDrag::canDecode(_event);
}

QListViewItem* ParsedUrlListView::currentDownloadItem(QListViewItem* parent, QString str_url)
{
// 	kdDebug(9815578)<<"ParsedUrlListView....currentDownloadItem..."<<endl;
	QListViewItem* curr_item = 0;
	
	if (parent != 0)
	{
		curr_item = parent->firstChild();
		while (curr_item != 0)
		{
			if (curr_item->text(0) == str_url)
			{
				break;
			}
			curr_item = curr_item->nextSibling();
		}
	}
	return curr_item;
}

QDragObject* ParsedUrlListView::dragObject()
{
	QPtrList<QListViewItem> selected_items = selectedItems();
	QStringList url_list;
	for ( QListViewItem* ite = selected_items.first(); ite; ite = selected_items.next() )
		url_list.append ( ite->text(0) );
	QString drag_text = url_list.join(",");
	return ( new QTextDrag(drag_text, this, "drag_item") );
}

void ParsedUrlListView::deselectAll()
{
	QListViewItem* curr_item = firstChild();
	while (curr_item != 0)
	{
		curr_item->setSelected( false );
		curr_item = curr_item->nextSibling();
	}
}

