/***************************************************************************
 *   Copyright (C) 2004 by Sayantam Dey                                    *
 *   sayantam@gmail.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef URLDISPLAY_H
#define URLDISPLAY_H

#include <klistview.h>
#include <kurl.h>
#include <kpopupmenu.h>

#include <qwidget.h>
#include <qlistview.h>
#include <qpixmap.h>
#include <qdragobject.h>
#include <qpainter.h>
#include <qmap.h>
#include <qvaluevector.h>

#include "krawler.h"
#include "infopanel.h"
#include "quicksetoptions.h"
#include "multijobhandler.h"
#include "qlayout.h"
#include "commonds.h"

/**
@author Sayantam Dey
*/


// re-implented for dnd
class ParsedUrlListView : public KListView
{
Q_OBJECT

public:
	ParsedUrlListView(QWidget* _parent = 0, const char* _name = 0, int _holder_width = 400);
	~ParsedUrlListView();
	bool acceptDrag(QDropEvent* _event) const;
	QListViewItem* currentDownloadItem(QListViewItem* parent, QString str_url);
	void deselectAll();
	QDragObject* dragObject();
};

class UrlDisplay : public QWidget
{
Q_OBJECT
public:
    UrlDisplay(const int _initial_width, QWidget *parent = 0, const char *name = 0);
    ~UrlDisplay();

	void addLinkInfo(int _index, QString q_str_url, int link_depth, const QStringList& _links, const QStringList& _images);
	void init_widget();
	void markCurrentDownload(QString str_url, int _url_type);
	void setInfoPanelInst(InfoPanel* _info_panel_inst);
	void setPageOptions(QuickSetOptions* _qso_options);
	void setMultiJobHandlerInst(MultiJobHandler* _mjh_inst);
	t_LeechJobList getCheckedUrls();	
	bool isCheckedItems();
	
public slots:
	void markCurrentDownload(uint, uint, int);
		
protected slots:
	void urlDropped(QDropEvent*);
	void slotCopyUrl(int, QString _sel_url_str);
	void slotUrlListContextMenu(KListView*, QListViewItem*, const QPoint&);
	void slotItemSelectionChanged();
	void slotUrlViewClicked();

protected:
	ParsedUrlListView* m_klvParsedUrls;
	QListViewItem* m_lastItem;
	QListViewItem* m_lastChild;
	QListViewItem* m_last_marked;
	KPopupMenu* url_list_menu_ptr;
	QPixmap m_image_pix;
	QPixmap m_html_pix;
	QPixmap m_mark_pix;
	InfoPanel* m_info_panel_inst;
	QuickSetOptions* m_qso_inst;
	MultiJobHandler* m_mjh_inst;	
	QVBoxLayout* q_layout;	
	QRect m_current_item_rect;
	int m_last_marked_pix_type;
	t_UrlRootItemList m_vec_url_root_items;
	int m_current_mode;
	
	//functions
	KPopupMenu* createUrlViewPopupMenu(KPopupMenu* _popup_menu_ptr);
	void debugMsg(QString _str);
	void markCurrentDownload(QListViewItem* _item, int _url_type);
		
	enum menu_ids
	{
		ID_COPY_URL = 1, ID_VIEW_SRC, ID_IGNORE_URL, ID_SELECT_URL
	};	  

private:
	
	enum link_type
	{
		ID_HTML = 21, ID_IMAGE
	};
	
signals:
	void ignoreLink(const QString& );

};

#endif
